@@ -1,12 +1,9 @@
-# -*- encoding: utf-8 -*-
 # This file is distributed under the same license as the Django package.
 #
-from __future__ import unicode_literals
-
 # The *_FORMAT strings use the Django date format syntax,
-# see http://docs.djangoproject.com/en/dev/ref/templates/builtins/#date
+# see https://docs.djangoproject.com/en/dev/ref/templates/builtins/#date
 DATE_FORMAT = 'j F Y'
-TIME_FORMAT = 'h:i:s A'
+TIME_FORMAT = 'h:i A'
 # DATETIME_FORMAT =
 # YEAR_MONTH_FORMAT =
 MONTH_DAY_FORMAT = 'j F'
@@ -15,7 +12,7 @@ SHORT_DATE_FORMAT = 'j M Y'
 # FIRST_DAY_OF_WEEK =
 
 # The *_INPUT_FORMATS strings use the Python strftime format syntax,
-# see http://docs.python.org/library/datetime.html#strftime-strptime-behavior
+# see https://docs.python.org/library/datetime.html#strftime-strptime-behavior
 # DATE_INPUT_FORMATS =
 # TIME_INPUT_FORMATS =
 # DATETIME_INPUT_FORMATS =
