@@ -2,6 +2,7 @@
 Management utility to create superusers.
 """
 import getpass
+import os
 import sys
 
 from django.contrib.auth import get_user_model
@@ -50,11 +51,28 @@ class Command(BaseCommand):
             default=DEFAULT_DB_ALIAS,
             help='Specifies the database to use. Default is "default".',
         )
-        for field in self.UserModel.REQUIRED_FIELDS:
-            parser.add_argument(
-                '--%s' % field,
-                help='Specifies the %s for the superuser.' % field,
-            )
+        for field_name in self.UserModel.REQUIRED_FIELDS:
+            field = self.UserModel._meta.get_field(field_name)
+            if field.many_to_many:
+                if field.remote_field.through and not field.remote_field.through._meta.auto_created:
+                    raise CommandError(
+                        "Required field '%s' specifies a many-to-many "
+                        "relation through model, which is not supported."
+                        % field_name
+                    )
+                else:
+                    parser.add_argument(
+                        '--%s' % field_name, action='append',
+                        help=(
+                            'Specifies the %s for the superuser. Can be used '
+                            'multiple times.' % field_name,
+                        ),
+                    )
+            else:
+                parser.add_argument(
+                    '--%s' % field_name,
+                    help='Specifies the %s for the superuser.' % field_name,
+                )
 
     def execute(self, *args, **options):
         self.stdin = options.get('stdin', sys.stdin)  # Used for testing
@@ -74,12 +92,12 @@ class Command(BaseCommand):
             user_data[PASSWORD_FIELD] = None
         try:
             if options['interactive']:
-                # Same as user_data but with foreign keys as fake model
-                # instances instead of raw IDs.
+                # Same as user_data but without many to many fields and with
+                # foreign keys as fake model instances instead of raw IDs.
                 fake_user_data = {}
                 if hasattr(self.stdin, 'isatty') and not self.stdin.isatty():
                     raise NotRunningInTTYException
-                default_username = get_default_username()
+                default_username = get_default_username(database=database)
                 if username:
                     error_msg = self._validate_username(username, verbose_field_name, database)
                     if error_msg:
@@ -110,10 +128,17 @@ class Command(BaseCommand):
                         message = self._get_input_message(field)
                         input_value = self.get_input_data(field, message)
                         user_data[field_name] = input_value
-                        fake_user_data[field_name] = input_value
+                        if field.many_to_many and input_value:
+                            if not input_value.strip():
+                                user_data[field_name] = None
+                                self.stderr.write('Error: This field cannot be blank.')
+                                continue
+                            user_data[field_name] = [pk.strip() for pk in input_value.split(',')]
+                        if not field.many_to_many:
+                            fake_user_data[field_name] = input_value
 
                         # Wrap any foreign keys in fake model instances
-                        if field.remote_field:
+                        if field.many_to_one:
                             fake_user_data[field_name] = field.remote_field.model(input_value)
 
                 # Prompt for a password if the model has one.
@@ -138,6 +163,13 @@ class Command(BaseCommand):
                     user_data[PASSWORD_FIELD] = password
             else:
                 # Non-interactive mode.
+                # Use password from environment variable, if provided.
+                if PASSWORD_FIELD in user_data and 'DJANGO_SUPERUSER_PASSWORD' in os.environ:
+                    user_data[PASSWORD_FIELD] = os.environ['DJANGO_SUPERUSER_PASSWORD']
+                # Use username from environment variable, if not provided in
+                # options.
+                if username is None:
+                    username = os.environ.get('DJANGO_SUPERUSER_' + self.UserModel.USERNAME_FIELD.upper())
                 if username is None:
                     raise CommandError('You must use --%s with --noinput.' % self.UserModel.USERNAME_FIELD)
                 else:
@@ -147,11 +179,12 @@ class Command(BaseCommand):
 
                 user_data[self.UserModel.USERNAME_FIELD] = username
                 for field_name in self.UserModel.REQUIRED_FIELDS:
-                    if options[field_name]:
-                        field = self.UserModel._meta.get_field(field_name)
-                        user_data[field_name] = field.clean(options[field_name], None)
-                    else:
+                    env_var = 'DJANGO_SUPERUSER_' + field_name.upper()
+                    value = options[field_name] or os.environ.get(env_var)
+                    if not value:
                         raise CommandError('You must use --%s with --noinput.' % field_name)
+                    field = self.UserModel._meta.get_field(field_name)
+                    user_data[field_name] = field.clean(value, None)
 
             self.UserModel._default_manager.db_manager(database).create_superuser(**user_data)
             if options['verbosity'] >= 1:
@@ -190,7 +223,7 @@ class Command(BaseCommand):
             " (leave blank to use '%s')" % default if default else '',
             ' (%s.%s)' % (
                 field.remote_field.model._meta.object_name,
-                field.remote_field.field_name,
+                field.m2m_target_field_name() if field.many_to_many else field.remote_field.field_name,
             ) if field.remote_field else '',
         )
 
