@@ -1,7 +1,7 @@
 import logging
 
-from django.contrib.gis.db.models.fields import GeometryField
-from django.db.utils import OperationalError
+from django.contrib.gis.db.models import GeometryField
+from django.db import OperationalError
 from django.db.backends.mysql.schema import DatabaseSchemaEditor
 
 logger = logging.getLogger('django.contrib.gis')
@@ -12,18 +12,18 @@ class MySQLGISSchemaEditor(DatabaseSchemaEditor):
     sql_drop_spatial_index = 'DROP INDEX %(index)s ON %(table)s'
 
     def __init__(self, *args, **kwargs):
-        super(MySQLGISSchemaEditor, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.geometry_sql = []
 
     def skip_default(self, field):
-        return (
-            super(MySQLGISSchemaEditor, self).skip_default(field) or
-            # Geometry fields are stored as BLOB/TEXT and can't have defaults.
-            isinstance(field, GeometryField)
-        )
+        # Geometry fields are stored as BLOB/TEXT, for which MySQL < 8.0.13 and
+        # MariaDB < 10.2.1 don't support defaults.
+        if isinstance(field, GeometryField) and not self._supports_limited_data_type_defaults:
+            return True
+        return super().skip_default(field)
 
     def column_sql(self, model, field, include_default=False):
-        column_sql = super(MySQLGISSchemaEditor, self).column_sql(model, field, include_default)
+        column_sql = super().column_sql(model, field, include_default)
         # MySQL doesn't support spatial indexes on NULL columns
         if isinstance(field, GeometryField) and field.spatial_index and not field.null:
             qn = self.connection.ops.quote_name
@@ -38,11 +38,11 @@ class MySQLGISSchemaEditor(DatabaseSchemaEditor):
         return column_sql
 
     def create_model(self, model):
-        super(MySQLGISSchemaEditor, self).create_model(model)
+        super().create_model(model)
         self.create_spatial_indexes()
 
     def add_field(self, model, field):
-        super(MySQLGISSchemaEditor, self).add_field(model, field)
+        super().add_field(model, field)
         self.create_spatial_indexes()
 
     def remove_field(self, model, field):
@@ -57,10 +57,10 @@ class MySQLGISSchemaEditor(DatabaseSchemaEditor):
             except OperationalError:
                 logger.error(
                     "Couldn't remove spatial index: %s (may be expected "
-                    "if your storage engine doesn't support them)." % sql
+                    "if your storage engine doesn't support them).", sql
                 )
 
-        super(MySQLGISSchemaEditor, self).remove_field(model, field)
+        super().remove_field(model, field)
 
     def _create_spatial_index_name(self, model, field):
         return '%s_%s_id' % (model._meta.db_table, field.column)
@@ -72,6 +72,6 @@ class MySQLGISSchemaEditor(DatabaseSchemaEditor):
             except OperationalError:
                 logger.error(
                     "Cannot create SPATIAL INDEX %s. Only MyISAM and (as of "
-                    "MySQL 5.7.5) InnoDB support them." % sql
+                    "MySQL 5.7.5) InnoDB support them.", sql
                 )
         self.geometry_sql = []
