@@ -1,12 +1,10 @@
-from __future__ import unicode_literals
-
 import logging
 import os
 import re
-from ctypes import c_char_p, c_int, CDLL, CFUNCTYPE
+from ctypes import CDLL, CFUNCTYPE, c_char_p, c_int
 from ctypes.util import find_library
 
-from django.contrib.gis.gdal.error import OGRException
+from django.contrib.gis.gdal.error import GDALException
 from django.core.exceptions import ImproperlyConfigured
 
 logger = logging.getLogger('django.contrib.gis')
@@ -15,21 +13,26 @@ logger = logging.getLogger('django.contrib.gis')
 try:
     from django.conf import settings
     lib_path = settings.GDAL_LIBRARY_PATH
-except (AttributeError, EnvironmentError,
-        ImportError, ImproperlyConfigured):
+except (AttributeError, ImportError, ImproperlyConfigured, OSError):
     lib_path = None
 
 if lib_path:
     lib_names = None
 elif os.name == 'nt':
     # Windows NT shared libraries
-    lib_names = ['gdal111', 'gdal110', 'gdal19', 'gdal18', 'gdal17', 'gdal16']
+    lib_names = [
+        'gdal302', 'gdal301', 'gdal300',
+        'gdal204', 'gdal203', 'gdal202', 'gdal201', 'gdal20',
+    ]
 elif os.name == 'posix':
     # *NIX library names.
-    lib_names = ['gdal', 'GDAL', 'gdal1.11.0', 'gdal1.10.0', 'gdal1.9.0',
-        'gdal1.8.0', 'gdal1.7.0', 'gdal1.6.0']
+    lib_names = [
+        'gdal', 'GDAL',
+        'gdal3.2.0', 'gdal3.1.0', 'gdal3.0.0',
+        'gdal2.4.0', 'gdal2.3.0', 'gdal2.2.0', 'gdal2.1.0', 'gdal2.0.0',
+    ]
 else:
-    raise OGRException('Unsupported OS "%s"' % os.name)
+    raise ImproperlyConfigured('GDAL is unsupported on OS "%s".' % os.name)
 
 # Using the ctypes `find_library` utility  to find the
 # path to the GDAL library from the list of library names.
@@ -40,9 +43,11 @@ if lib_names:
             break
 
 if lib_path is None:
-    raise OGRException('Could not find the GDAL library (tried "%s"). '
-                       'Try setting GDAL_LIBRARY_PATH in your settings.' %
-                       '", "'.join(lib_names))
+    raise ImproperlyConfigured(
+        'Could not find the GDAL library (tried "%s"). Is GDAL installed? '
+        'If it is, try setting GDAL_LIBRARY_PATH in your settings.'
+        % '", "'.join(lib_names)
+    )
 
 # This loads the GDAL/OGR C library
 lgdal = CDLL(lib_path)
@@ -58,7 +63,7 @@ if os.name == 'nt':
 
 def std_call(func):
     """
-    Returns the correct STDCALL function for certain OSR routines on Win32
+    Return the correct STDCALL function for certain OSR routines on Win32
     platforms.
     """
     if os.name == 'nt':
@@ -66,46 +71,44 @@ def std_call(func):
     else:
         return lgdal[func]
 
-#### Version-information functions. ####
 
-# Returns GDAL library version information with the given key.
+# #### Version-information functions. ####
+
+# Return GDAL library version information with the given key.
 _version_info = std_call('GDALVersionInfo')
 _version_info.argtypes = [c_char_p]
 _version_info.restype = c_char_p
 
 
 def gdal_version():
-    "Returns only the GDAL version number information."
+    "Return only the GDAL version number information."
     return _version_info(b'RELEASE_NAME')
 
 
 def gdal_full_version():
-    "Returns the full GDAL version information."
-    return _version_info('')
-
-version_regex = re.compile(r'^(?P<major>\d+)\.(?P<minor>\d+)(\.(?P<subminor>\d+))?')
+    "Return the full GDAL version information."
+    return _version_info(b'')
 
 
 def gdal_version_info():
-    ver = gdal_version().decode()
-    m = version_regex.match(ver)
+    ver = gdal_version()
+    m = re.match(br'^(?P<major>\d+)\.(?P<minor>\d+)(?:\.(?P<subminor>\d+))?', ver)
     if not m:
-        raise OGRException('Could not parse GDAL version string "%s"' % ver)
-    return dict((key, m.group(key)) for key in ('major', 'minor', 'subminor'))
+        raise GDALException('Could not parse GDAL version string "%s"' % ver)
+    major, minor, subminor = m.groups()
+    return (int(major), int(minor), subminor and int(subminor))
 
-_verinfo = gdal_version_info()
-GDAL_MAJOR_VERSION = int(_verinfo['major'])
-GDAL_MINOR_VERSION = int(_verinfo['minor'])
-GDAL_SUBMINOR_VERSION = _verinfo['subminor'] and int(_verinfo['subminor'])
-GDAL_VERSION = (GDAL_MAJOR_VERSION, GDAL_MINOR_VERSION, GDAL_SUBMINOR_VERSION)
-del _verinfo
+
+GDAL_VERSION = gdal_version_info()
 
 # Set library error handling so as errors are logged
 CPLErrorHandler = CFUNCTYPE(None, c_int, c_int, c_char_p)
 
 
 def err_handler(error_class, error_number, message):
-    logger.error('GDAL_ERROR %d: %s' % (error_number, message))
+    logger.error('GDAL_ERROR %d: %s', error_number, message)
+
+
 err_handler = CPLErrorHandler(err_handler)
 
 
@@ -115,5 +118,6 @@ def function(name, args, restype):
     func.restype = restype
     return func
 
+
 set_error_handler = function('CPLSetErrorHandler', [CPLErrorHandler], CPLErrorHandler)
 set_error_handler(err_handler)
