@@ -1,18 +1,18 @@
 from django.contrib.gis.gdal import SpatialReference
+from django.db import DEFAULT_DB_ALIAS, connections
+
 
 def add_srs_entry(srs, auth_name='EPSG', auth_srid=None, ref_sys_name=None,
                   database=None):
     """
-    This function takes a GDAL SpatialReference system and adds its information
-    to the `spatial_ref_sys` table of the spatial backend.  Doing this enables
+    Take a GDAL SpatialReference system and add its information to the
+    `spatial_ref_sys` table of the spatial backend. Doing this enables
     database-level spatial transformations for the backend.  Thus, this utility
     is useful for adding spatial reference systems not included by default with
-    the backend -- for example, the so-called "Google Maps Mercator Projection"
-    is excluded in PostGIS 1.3 and below, and the following adds it to the
-    `spatial_ref_sys` table:
+    the backend:
 
     >>> from django.contrib.gis.utils import add_srs_entry
-    >>> add_srs_entry(900913)
+    >>> add_srs_entry(3857)
 
     Keyword Arguments:
      auth_name:
@@ -29,20 +29,17 @@ def add_srs_entry(srs, auth_name='EPSG', auth_srid=None, ref_sys_name=None,
 
      database:
       The name of the database connection to use; the default is the value
-      of `django.db.DEFAULT_DB_ALIAS` (at the time of this writing, it's value
+      of `django.db.DEFAULT_DB_ALIAS` (at the time of this writing, its value
       is 'default').
     """
-    from django.db import connections, DEFAULT_DB_ALIAS
-    if not database:
-        database = DEFAULT_DB_ALIAS
+    database = database or DEFAULT_DB_ALIAS
     connection = connections[database]
 
     if not hasattr(connection.ops, 'spatial_version'):
         raise Exception('The `add_srs_entry` utility only works '
                         'with spatial backends.')
-    if connection.ops.oracle or connection.ops.mysql:
-        raise Exception('This utility does not support the '
-                        'Oracle or MySQL spatial backends.')
+    if not connection.features.supports_add_srs_entry:
+        raise Exception('This utility does not support your database backend.')
     SpatialRefSys = connection.ops.spatial_ref_sys()
 
     # If argument is not a `SpatialReference` instance, use it as parameter
@@ -56,25 +53,24 @@ def add_srs_entry(srs, auth_name='EPSG', auth_srid=None, ref_sys_name=None,
 
     # Initializing the keyword arguments dictionary for both PostGIS
     # and SpatiaLite.
-    kwargs = {'srid' : srs.srid,
-              'auth_name' : auth_name,
-              'auth_srid' : auth_srid or srs.srid,
-              'proj4text' : srs.proj4,
-              }
-
+    kwargs = {
+        'srid': srs.srid,
+        'auth_name': auth_name,
+        'auth_srid': auth_srid or srs.srid,
+        'proj4text': srs.proj4,
+    }
     # Backend-specific fields for the SpatialRefSys model.
-    if connection.ops.postgis:
+    srs_field_names = {f.name for f in SpatialRefSys._meta.get_fields()}
+    if 'srtext' in srs_field_names:
         kwargs['srtext'] = srs.wkt
-    if connection.ops.spatialite:
+    if 'ref_sys_name' in srs_field_names:
+        # SpatiaLite specific
         kwargs['ref_sys_name'] = ref_sys_name or srs.name
 
     # Creating the spatial_ref_sys model.
     try:
         # Try getting via SRID only, because using all kwargs may
         # differ from exact wkt/proj in database.
-        sr = SpatialRefSys.objects.using(database).get(srid=srs.srid)
+        SpatialRefSys.objects.using(database).get(srid=srs.srid)
     except SpatialRefSys.DoesNotExist:
-        sr = SpatialRefSys.objects.using(database).create(**kwargs)
-
-# Alias is for backwards-compatibility purposes.
-add_postgis_srs = add_srs_entry
+        SpatialRefSys.objects.using(database).create(**kwargs)
