@@ -1,7 +1,9 @@
+import binascii
 import json
 
 from django.conf import settings
 from django.contrib.messages.storage.base import BaseStorage, Message
+from django.core import signing
 from django.http import SimpleCookie
 from django.utils.crypto import constant_time_compare, salted_hmac
 from django.utils.safestring import SafeData, mark_safe
@@ -32,9 +34,6 @@ class MessageDecoder(json.JSONDecoder):
     def process_messages(self, obj):
         if isinstance(obj, list) and obj:
             if obj[0] == MessageEncoder.message_key:
-                if len(obj) == 3:
-                    # Compatibility with previously-encoded messages
-                    return Message(*obj[1:])
                 if obj[1]:
                     obj[3] = mark_safe(obj[3])
                 return Message(*obj[2:])
@@ -49,6 +48,18 @@ class MessageDecoder(json.JSONDecoder):
         return self.process_messages(decoded)
 
 
+class MessageSerializer:
+    def dumps(self, obj):
+        return json.dumps(
+            obj,
+            separators=(',', ':'),
+            cls=MessageEncoder,
+        ).encode('latin-1')
+
+    def loads(self, data):
+        return json.loads(data.decode('latin-1'), cls=MessageDecoder)
+
+
 class CookieStorage(BaseStorage):
     """
     Store messages in a cookie.
@@ -59,6 +70,11 @@ class CookieStorage(BaseStorage):
     # restrict the session cookie to 1/2 of 4kb. See #18781.
     max_cookie_size = 2048
     not_finished = '__messagesnotfinished__'
+    key_salt = 'django.contrib.messages'
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.signer = signing.get_cookie_signer(salt=self.key_salt)
 
     def _get(self, *args, **kwargs):
         """
@@ -124,11 +140,15 @@ class CookieStorage(BaseStorage):
         self._update_cookie(encoded_data, response)
         return unstored_messages
 
-    def _hash(self, value):
+    def _legacy_hash(self, value):
         """
+        # RemovedInDjango40Warning: pre-Django 3.1 hashes will be invalid.
         Create an HMAC/SHA1 hash based on the value and the project setting's
         SECRET_KEY, modified to make it unique for the present purpose.
         """
+        # The class wide key salt is not reused here since older Django
+        # versions had it fixed and making it dynamic would break old hashes if
+        # self.key_salt is changed.
         key_salt = 'django.contrib.messages'
         return salted_hmac(key_salt, value).hexdigest()
 
@@ -141,9 +161,7 @@ class CookieStorage(BaseStorage):
         also contains a hash to ensure that the data was not tampered with.
         """
         if messages or encode_empty:
-            encoder = MessageEncoder(separators=(',', ':'))
-            value = encoder.encode(messages)
-            return '%s$%s' % (self._hash(value), value)
+            return self.signer.sign_object(messages, serializer=MessageSerializer, compress=True)
 
     def _decode(self, data):
         """
@@ -154,17 +172,36 @@ class CookieStorage(BaseStorage):
         """
         if not data:
             return None
-        bits = data.split('$', 1)
-        if len(bits) == 2:
-            hash, value = bits
-            if constant_time_compare(hash, self._hash(value)):
-                try:
-                    # If we get here (and the JSON decode works), everything is
-                    # good. In any other case, drop back and return None.
-                    return json.loads(value, cls=MessageDecoder)
-                except json.JSONDecodeError:
-                    pass
+        try:
+            return self.signer.unsign_object(data, serializer=MessageSerializer)
+        # RemovedInDjango41Warning: when the deprecation ends, replace with:
+        #
+        # except (signing.BadSignature, json.JSONDecodeError):
+        #     pass
+        except signing.BadSignature:
+            # RemovedInDjango40Warning: when the deprecation ends, replace
+            # with:
+            #   decoded = None.
+            decoded = self._legacy_decode(data)
+        except (binascii.Error, json.JSONDecodeError):
+            decoded = self.signer.unsign(data)
+
+        if decoded:
+            # RemovedInDjango41Warning.
+            try:
+                return json.loads(decoded, cls=MessageDecoder)
+            except json.JSONDecodeError:
+                pass
         # Mark the data as used (so it gets removed) since something was wrong
         # with the data.
         self.used = True
         return None
+
+    def _legacy_decode(self, data):
+        # RemovedInDjango40Warning: pre-Django 3.1 hashes will be invalid.
+        bits = data.split('$', 1)
+        if len(bits) == 2:
+            hash_, value = bits
+            if constant_time_compare(hash_, self._legacy_hash(value)):
+                return value
+        return None
