@@ -1,6 +1,5 @@
 import json
 
-from django.conf import settings
 from django.contrib.messages.storage.base import BaseStorage
 from django.contrib.messages.storage.cookie import (
     MessageDecoder, MessageEncoder,
@@ -17,7 +16,7 @@ class SessionStorage(BaseStorage):
         assert hasattr(request, 'session'), "The session-based temporary "\
             "message storage requires session middleware to be installed, "\
             "and come before the message middleware in the "\
-            "MIDDLEWARE%s list." % ("_CLASSES" if settings.MIDDLEWARE is None else "")
+            "MIDDLEWARE list."
         super().__init__(request, *args, **kwargs)
 
     def _get(self, *args, **kwargs):
@@ -39,7 +38,7 @@ class SessionStorage(BaseStorage):
         return []
 
     def serialize_messages(self, messages):
-        encoder = MessageEncoder(separators=(',', ':'))
+        encoder = MessageEncoder()
         return encoder.encode(messages)
 
     def deserialize_messages(self, data):
