@@ -1,10 +1,10 @@
-from django.db.models import Index
-from django.db.utils import NotSupportedError
+from django.db import NotSupportedError
+from django.db.models import Func, Index
 from django.utils.functional import cached_property
 
 __all__ = [
-    'BrinIndex', 'BTreeIndex', 'GinIndex', 'GistIndex', 'HashIndex',
-    'SpGistIndex',
+    'BloomIndex', 'BrinIndex', 'BTreeIndex', 'GinIndex', 'GistIndex',
+    'HashIndex', 'SpGistIndex',
 ]
 
 
@@ -18,9 +18,9 @@ class PostgresIndex(Index):
         # indexes.
         return Index.max_name_length - len(Index.suffix) + len(self.suffix)
 
-    def create_sql(self, model, schema_editor, using=''):
+    def create_sql(self, model, schema_editor, using='', **kwargs):
         self.check_supported(schema_editor)
-        statement = super().create_sql(model, schema_editor, using=' USING %s' % self.suffix)
+        statement = super().create_sql(model, schema_editor, using=' USING %s' % self.suffix, **kwargs)
         with_params = self.get_with_params()
         if with_params:
             statement.parts['extra'] = 'WITH (%s) %s' % (
@@ -36,15 +36,59 @@ class PostgresIndex(Index):
         return []
 
 
+class BloomIndex(PostgresIndex):
+    suffix = 'bloom'
+
+    def __init__(self, *expressions, length=None, columns=(), **kwargs):
+        super().__init__(*expressions, **kwargs)
+        if len(self.fields) > 32:
+            raise ValueError('Bloom indexes support a maximum of 32 fields.')
+        if not isinstance(columns, (list, tuple)):
+            raise ValueError('BloomIndex.columns must be a list or tuple.')
+        if len(columns) > len(self.fields):
+            raise ValueError(
+                'BloomIndex.columns cannot have more values than fields.'
+            )
+        if not all(0 < col <= 4095 for col in columns):
+            raise ValueError(
+                'BloomIndex.columns must contain integers from 1 to 4095.',
+            )
+        if length is not None and not 0 < length <= 4096:
+            raise ValueError(
+                'BloomIndex.length must be None or an integer from 1 to 4096.',
+            )
+        self.length = length
+        self.columns = columns
+
+    def deconstruct(self):
+        path, args, kwargs = super().deconstruct()
+        if self.length is not None:
+            kwargs['length'] = self.length
+        if self.columns:
+            kwargs['columns'] = self.columns
+        return path, args, kwargs
+
+    def get_with_params(self):
+        with_params = []
+        if self.length is not None:
+            with_params.append('length = %d' % self.length)
+        if self.columns:
+            with_params.extend(
+                'col%d = %d' % (i, v)
+                for i, v in enumerate(self.columns, start=1)
+            )
+        return with_params
+
+
 class BrinIndex(PostgresIndex):
     suffix = 'brin'
 
-    def __init__(self, *, autosummarize=None, pages_per_range=None, **kwargs):
+    def __init__(self, *expressions, autosummarize=None, pages_per_range=None, **kwargs):
         if pages_per_range is not None and pages_per_range <= 0:
             raise ValueError('pages_per_range must be None or a positive integer')
         self.autosummarize = autosummarize
         self.pages_per_range = pages_per_range
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -55,8 +99,6 @@ class BrinIndex(PostgresIndex):
         return path, args, kwargs
 
     def check_supported(self, schema_editor):
-        if not schema_editor.connection.features.has_brin_index_support:
-            raise NotSupportedError('BRIN indexes require PostgreSQL 9.5+.')
         if self.autosummarize and not schema_editor.connection.features.has_brin_autosummarize:
             raise NotSupportedError('BRIN option autosummarize requires PostgreSQL 10+.')
 
@@ -72,9 +114,9 @@ class BrinIndex(PostgresIndex):
 class BTreeIndex(PostgresIndex):
     suffix = 'btree'
 
-    def __init__(self, *, fillfactor=None, **kwargs):
+    def __init__(self, *expressions, fillfactor=None, **kwargs):
         self.fillfactor = fillfactor
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -92,10 +134,10 @@ class BTreeIndex(PostgresIndex):
 class GinIndex(PostgresIndex):
     suffix = 'gin'
 
-    def __init__(self, *, fastupdate=None, gin_pending_list_limit=None, **kwargs):
+    def __init__(self, *expressions, fastupdate=None, gin_pending_list_limit=None, **kwargs):
         self.fastupdate = fastupdate
         self.gin_pending_list_limit = gin_pending_list_limit
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -105,10 +147,6 @@ class GinIndex(PostgresIndex):
             kwargs['gin_pending_list_limit'] = self.gin_pending_list_limit
         return path, args, kwargs
 
-    def check_supported(self, schema_editor):
-        if self.gin_pending_list_limit and not schema_editor.connection.features.has_gin_pending_list_limit:
-            raise NotSupportedError('GIN option gin_pending_list_limit requires PostgreSQL 9.5+.')
-
     def get_with_params(self):
         with_params = []
         if self.gin_pending_list_limit is not None:
@@ -121,10 +159,10 @@ class GinIndex(PostgresIndex):
 class GistIndex(PostgresIndex):
     suffix = 'gist'
 
-    def __init__(self, *, buffering=None, fillfactor=None, **kwargs):
+    def __init__(self, *expressions, buffering=None, fillfactor=None, **kwargs):
         self.buffering = buffering
         self.fillfactor = fillfactor
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -142,13 +180,17 @@ class GistIndex(PostgresIndex):
             with_params.append('fillfactor = %d' % self.fillfactor)
         return with_params
 
+    def check_supported(self, schema_editor):
+        if self.include and not schema_editor.connection.features.supports_covering_gist_indexes:
+            raise NotSupportedError('Covering GiST indexes requires PostgreSQL 12+.')
+
 
 class HashIndex(PostgresIndex):
     suffix = 'hash'
 
-    def __init__(self, *, fillfactor=None, **kwargs):
+    def __init__(self, *expressions, fillfactor=None, **kwargs):
         self.fillfactor = fillfactor
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -166,9 +208,9 @@ class HashIndex(PostgresIndex):
 class SpGistIndex(PostgresIndex):
     suffix = 'spgist'
 
-    def __init__(self, *, fillfactor=None, **kwargs):
+    def __init__(self, *expressions, fillfactor=None, **kwargs):
         self.fillfactor = fillfactor
-        super().__init__(**kwargs)
+        super().__init__(*expressions, **kwargs)
 
     def deconstruct(self):
         path, args, kwargs = super().deconstruct()
@@ -181,3 +223,10 @@ class SpGistIndex(PostgresIndex):
         if self.fillfactor is not None:
             with_params.append('fillfactor = %d' % self.fillfactor)
         return with_params
+
+
+class OpClass(Func):
+    template = '%(expressions)s %(name)s'
+
+    def __init__(self, expression, name):
+        super().__init__(expression, name=name)
