@@ -1,19 +1,20 @@
-from __future__ import unicode_literals
-
 import base64
 import logging
 import string
+import warnings
 from datetime import datetime, timedelta
 
 from django.conf import settings
 from django.contrib.sessions.exceptions import SuspiciousSession
+from django.core import signing
 from django.core.exceptions import SuspiciousOperation
 from django.utils import timezone
 from django.utils.crypto import (
     constant_time_compare, get_random_string, salted_hmac,
 )
-from django.utils.encoding import force_bytes, force_text
+from django.utils.deprecation import RemovedInDjango40Warning
 from django.utils.module_loading import import_string
+from django.utils.translation import LANGUAGE_SESSION_KEY
 
 # session_key should not be case sensitive because some backends can store it
 # on case insensitive file systems.
@@ -28,13 +29,22 @@ class CreateError(Exception):
     pass
 
 
-class SessionBase(object):
+class UpdateError(Exception):
+    """
+    Occurs if Django tries to update a session that was deleted.
+    """
+    pass
+
+
+class SessionBase:
     """
     Base class for all Session classes.
     """
     TEST_COOKIE_NAME = 'testcookie'
     TEST_COOKIE_VALUE = 'worked'
 
+    __not_given = object()
+
     def __init__(self, session_key=None):
         self._session_key = session_key
         self.accessed = False
@@ -45,6 +55,13 @@ class SessionBase(object):
         return key in self._session
 
     def __getitem__(self, key):
+        if key == LANGUAGE_SESSION_KEY:
+            warnings.warn(
+                'The user language will no longer be stored in '
+                'request.session in Django 4.0. Read it from '
+                'request.COOKIES[settings.LANGUAGE_COOKIE_NAME] instead.',
+                RemovedInDjango40Warning, stacklevel=2,
+            )
         return self._session[key]
 
     def __setitem__(self, key, value):
@@ -55,11 +72,16 @@ class SessionBase(object):
         del self._session[key]
         self.modified = True
 
+    @property
+    def key_salt(self):
+        return 'django.contrib.sessions.' + self.__class__.__qualname__
+
     def get(self, key, default=None):
         return self._session.get(key, default)
 
-    def pop(self, key, *args):
+    def pop(self, key, default=__not_given):
         self.modified = self.modified or key in self._session
+        args = () if default is self.__not_given else (default,)
         return self._session.pop(key, *args)
 
     def setdefault(self, key, value):
@@ -80,17 +102,46 @@ class SessionBase(object):
         del self[self.TEST_COOKIE_NAME]
 
     def _hash(self, value):
+        # RemovedInDjango40Warning: pre-Django 3.1 format will be invalid.
         key_salt = "django.contrib.sessions" + self.__class__.__name__
         return salted_hmac(key_salt, value).hexdigest()
 
     def encode(self, session_dict):
-        "Returns the given session dictionary serialized and encoded as a string."
-        serialized = self.serializer().dumps(session_dict)
-        hash = self._hash(serialized)
-        return base64.b64encode(hash.encode() + b":" + serialized).decode('ascii')
+        "Return the given session dictionary serialized and encoded as a string."
+        # RemovedInDjango40Warning: DEFAULT_HASHING_ALGORITHM will be removed.
+        if settings.DEFAULT_HASHING_ALGORITHM == 'sha1':
+            return self._legacy_encode(session_dict)
+        return signing.dumps(
+            session_dict, salt=self.key_salt, serializer=self.serializer,
+            compress=True,
+        )
 
     def decode(self, session_data):
-        encoded_data = base64.b64decode(force_bytes(session_data))
+        try:
+            return signing.loads(session_data, salt=self.key_salt, serializer=self.serializer)
+        # RemovedInDjango40Warning: when the deprecation ends, handle here
+        # exceptions similar to what _legacy_decode() does now.
+        except signing.BadSignature:
+            try:
+                # Return an empty session if data is not in the pre-Django 3.1
+                # format.
+                return self._legacy_decode(session_data)
+            except Exception:
+                logger = logging.getLogger('django.security.SuspiciousSession')
+                logger.warning('Session data corrupted')
+                return {}
+        except Exception:
+            return self._legacy_decode(session_data)
+
+    def _legacy_encode(self, session_dict):
+        # RemovedInDjango40Warning.
+        serialized = self.serializer().dumps(session_dict)
+        hash = self._hash(serialized)
+        return base64.b64encode(hash.encode() + b':' + serialized).decode('ascii')
+
+    def _legacy_decode(self, session_data):
+        # RemovedInDjango40Warning: pre-Django 3.1 format will be invalid.
+        encoded_data = base64.b64decode(session_data.encode('ascii'))
         try:
             # could produce ValueError if there is no ':'
             hash, serialized = encoded_data.split(b':', 1)
@@ -103,9 +154,8 @@ class SessionBase(object):
             # ValueError, SuspiciousOperation, unpickling exceptions. If any of
             # these happen, just return an empty dictionary (an empty session).
             if isinstance(e, SuspiciousOperation):
-                logger = logging.getLogger('django.security.%s' %
-                        e.__class__.__name__)
-                logger.warning(force_text(e))
+                logger = logging.getLogger('django.security.%s' % e.__class__.__name__)
+                logger.warning(str(e))
             return {}
 
     def update(self, dict_):
@@ -124,15 +174,6 @@ class SessionBase(object):
     def items(self):
         return self._session.items()
 
-    def iterkeys(self):
-        return self._session.iterkeys()
-
-    def itervalues(self):
-        return self._session.itervalues()
-
-    def iteritems(self):
-        return self._session.iteritems()
-
     def clear(self):
         # To avoid unnecessary persistent storage accesses, we set up the
         # internals directly (loading data wastes time, since we are going to
@@ -142,34 +183,50 @@ class SessionBase(object):
         self.modified = True
 
     def is_empty(self):
-        "Returns True when there is no session_key and the session is empty"
+        "Return True when there is no session_key and the session is empty."
         try:
-            return not bool(self._session_key) and not self._session_cache
+            return not self._session_key and not self._session_cache
         except AttributeError:
             return True
 
     def _get_new_session_key(self):
-        "Returns session key that isn't being used."
+        "Return session key that isn't being used."
         while True:
             session_key = get_random_string(32, VALID_KEY_CHARS)
             if not self.exists(session_key):
-                break
-        return session_key
+                return session_key
 
     def _get_or_create_session_key(self):
         if self._session_key is None:
             self._session_key = self._get_new_session_key()
         return self._session_key
 
+    def _validate_session_key(self, key):
+        """
+        Key must be truthy and at least 8 characters long. 8 characters is an
+        arbitrary lower bound for some minimal key security.
+        """
+        return key and len(key) >= 8
+
     def _get_session_key(self):
-        return self._session_key
+        return self.__session_key
+
+    def _set_session_key(self, value):
+        """
+        Validate session key on assignment. Invalid values will set to None.
+        """
+        if self._validate_session_key(value):
+            self.__session_key = value
+        else:
+            self.__session_key = None
 
     session_key = property(_get_session_key)
+    _session_key = property(_get_session_key, _set_session_key)
 
     def _get_session(self, no_load=False):
         """
-        Lazily loads session from storage (unless "no_load" is True, when only
-        an empty dict is stored) and stores it in the current instance.
+        Lazily load session from storage (unless "no_load" is True, when only
+        an empty dict is stored) and store it in the current instance.
         """
         self.accessed = True
         try:
@@ -183,6 +240,9 @@ class SessionBase(object):
 
     _session = property(_get_session)
 
+    def get_session_cookie_age(self):
+        return settings.SESSION_COOKIE_AGE
+
     def get_expiry_age(self, **kwargs):
         """Get the number of seconds until the session expires.
 
@@ -202,7 +262,7 @@ class SessionBase(object):
             expiry = self.get('_session_expiry')
 
         if not expiry:   # Checks both None and 0 cases
-            return settings.SESSION_COOKIE_AGE
+            return self.get_session_cookie_age()
         if not isinstance(expiry, datetime):
             return expiry
         delta = expiry - modification
@@ -226,13 +286,12 @@ class SessionBase(object):
 
         if isinstance(expiry, datetime):
             return expiry
-        if not expiry:   # Checks both None and 0 cases
-            expiry = settings.SESSION_COOKIE_AGE
+        expiry = expiry or self.get_session_cookie_age()
         return modification + timedelta(seconds=expiry)
 
     def set_expiry(self, value):
         """
-        Sets a custom expiration for the session. ``value`` can be an integer,
+        Set a custom expiration for the session. ``value`` can be an integer,
         a Python ``datetime`` or ``timedelta`` object or ``None``.
 
         If ``value`` is an integer, the session will expire after that many
@@ -258,7 +317,7 @@ class SessionBase(object):
 
     def get_expire_at_browser_close(self):
         """
-        Returns ``True`` if the session is set to expire when the browser
+        Return ``True`` if the session is set to expire when the browser
         closes, and ``False`` if there's an expiry date. Use
         ``get_expiry_date()`` or ``get_expiry_age()`` to find the actual expiry
         date/age, if there is one.
@@ -269,7 +328,7 @@ class SessionBase(object):
 
     def flush(self):
         """
-        Removes the current session data from the database and regenerates the
+        Remove the current session data from the database and regenerate the
         key.
         """
         self.clear()
@@ -278,25 +337,26 @@ class SessionBase(object):
 
     def cycle_key(self):
         """
-        Creates a new session key, whilst retaining the current session data.
+        Create a new session key, while retaining the current session data.
         """
-        data = self._session_cache
+        data = self._session
         key = self.session_key
         self.create()
         self._session_cache = data
-        self.delete(key)
+        if key:
+            self.delete(key)
 
     # Methods that child classes must implement.
 
     def exists(self, session_key):
         """
-        Returns True if the given session_key already exists.
+        Return True if the given session_key already exists.
         """
         raise NotImplementedError('subclasses of SessionBase must provide an exists() method')
 
     def create(self):
         """
-        Creates a new session instance. Guaranteed to create a new object with
+        Create a new session instance. Guaranteed to create a new object with
         a unique key and will have saved the result once (with empty data)
         before the method returns.
         """
@@ -304,22 +364,22 @@ class SessionBase(object):
 
     def save(self, must_create=False):
         """
-        Saves the session data. If 'must_create' is True, a new session object
-        is created (otherwise a CreateError exception is raised). Otherwise,
-        save() can update an existing object with the same key.
+        Save the session data. If 'must_create' is True, create a new session
+        object (or raise CreateError). Otherwise, only update an existing
+        object and don't create one (raise UpdateError if needed).
         """
         raise NotImplementedError('subclasses of SessionBase must provide a save() method')
 
     def delete(self, session_key=None):
         """
-        Deletes the session data under this key. If the key is None, the
-        current session key value is used.
+        Delete the session data under this key. If the key is None, use the
+        current session key value.
         """
         raise NotImplementedError('subclasses of SessionBase must provide a delete() method')
 
     def load(self):
         """
-        Loads the session data and returns a dictionary.
+        Load the session data and return a dictionary.
         """
         raise NotImplementedError('subclasses of SessionBase must provide a load() method')
 
