@@ -44,13 +44,13 @@ class Command(BaseCommand):
             except ImportError:
                 pass
 
-        sql_list = sql_flush(self.style, connection, only_django=True,
+        sql_list = sql_flush(self.style, connection,
                              reset_sequences=reset_sequences,
                              allow_cascade=allow_cascade)
 
         if interactive:
             confirm = input("""You have requested a flush of the database.
-This will IRREVERSIBLY DESTROY all data currently in the %r database,
+This will IRREVERSIBLY DESTROY all data currently in the "%s" database,
 and return each table to an empty state.
 Are you sure you want to do this?
 
@@ -60,7 +60,7 @@ Are you sure you want to do this?
 
         if confirm == 'yes':
             try:
-                connection.ops.execute_sql_flush(database, sql_list)
+                connection.ops.execute_sql_flush(sql_list)
             except Exception as exc:
                 raise CommandError(
                     "Database %s couldn't be flushed. Possible reasons:\n"
@@ -68,7 +68,7 @@ Are you sure you want to do this?
                     "  * At least one of the expected database tables doesn't exist.\n"
                     "  * The SQL was invalid.\n"
                     "Hint: Look at the output of 'django-admin sqlflush'. "
-                    "That's the SQL this command wasn't able to run.\n" % (
+                    "That's the SQL this command wasn't able to run." % (
                         connection.settings_dict['NAME'],
                     )
                 ) from exc
@@ -79,4 +79,4 @@ Are you sure you want to do this?
                 # respond as if the database had been migrated from scratch.
                 emit_post_migrate_signal(verbosity, interactive, database)
         else:
-            self.stdout.write("Flush cancelled.\n")
+            self.stdout.write('Flush cancelled.')
