@@ -1,5 +1,6 @@
+import uuid
+
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class Event(models.Model):
@@ -12,7 +13,7 @@ class Parent(models.Model):
 
 
 class Child(models.Model):
-    parent = models.ForeignKey(Parent, editable=False, null=True)
+    parent = models.ForeignKey(Parent, models.SET_NULL, editable=False, null=True)
     name = models.CharField(max_length=30, blank=True)
     age = models.IntegerField(null=True, blank=True)
 
@@ -27,15 +28,14 @@ class Band(models.Model):
     genres = models.ManyToManyField(Genre)
 
 
-@python_2_unicode_compatible
 class Musician(models.Model):
     name = models.CharField(max_length=30)
+    age = models.IntegerField(null=True, blank=True)
 
     def __str__(self):
         return self.name
 
 
-@python_2_unicode_compatible
 class Group(models.Model):
     name = models.CharField(max_length=30)
     members = models.ManyToManyField(Musician, through='Membership')
@@ -44,9 +44,14 @@ class Group(models.Model):
         return self.name
 
 
+class Concert(models.Model):
+    name = models.CharField(max_length=30)
+    group = models.ForeignKey(Group, models.CASCADE)
+
+
 class Membership(models.Model):
-    music = models.ForeignKey(Musician)
-    group = models.ForeignKey(Group)
+    music = models.ForeignKey(Musician, models.CASCADE)
+    group = models.ForeignKey(Group, models.CASCADE)
     role = models.CharField(max_length=15)
 
 
@@ -64,12 +69,13 @@ class ChordsBand(models.Model):
 
 
 class Invitation(models.Model):
-    player = models.ForeignKey(ChordsMusician)
-    band = models.ForeignKey(ChordsBand)
+    player = models.ForeignKey(ChordsMusician, models.CASCADE)
+    band = models.ForeignKey(ChordsBand, models.CASCADE)
     instrument = models.CharField(max_length=15)
 
 
 class Swallow(models.Model):
+    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4)
     origin = models.CharField(max_length=255)
     load = models.FloatField()
     speed = models.FloatField()
@@ -79,7 +85,7 @@ class Swallow(models.Model):
 
 
 class SwallowOneToOne(models.Model):
-    swallow = models.OneToOneField(Swallow)
+    swallow = models.OneToOneField(Swallow, models.CASCADE)
 
 
 class UnorderedObject(models.Model):
@@ -92,7 +98,7 @@ class UnorderedObject(models.Model):
 
 class OrderedObjectManager(models.Manager):
     def get_queryset(self):
-        return super(OrderedObjectManager, self).get_queryset().order_by('number')
+        return super().get_queryset().order_by('number')
 
 
 class OrderedObject(models.Model):
@@ -109,3 +115,7 @@ class OrderedObject(models.Model):
 
 class CustomIdUser(models.Model):
     uuid = models.AutoField(primary_key=True)
+
+
+class CharPK(models.Model):
+    char_pk = models.CharField(max_length=100, primary_key=True)
