@@ -1,4 +1,5 @@
 from django.contrib.auth.models import User
+from django.test import override_settings
 
 from .tests import AdminDocsTestCase, TestDataMixin
 
@@ -12,7 +13,7 @@ class XViewMiddlewareTest(TestDataMixin, AdminDocsTestCase):
         self.client.force_login(self.superuser)
         response = self.client.head('/xview/func/')
         self.assertIn('X-View', response)
-        self.assertEqual(response['X-View'], 'admin_docs.views.xview')
+        self.assertEqual(response.headers['X-View'], 'admin_docs.views.xview')
         user.is_staff = False
         user.save()
         response = self.client.head('/xview/func/')
@@ -30,7 +31,7 @@ class XViewMiddlewareTest(TestDataMixin, AdminDocsTestCase):
         self.client.force_login(self.superuser)
         response = self.client.head('/xview/class/')
         self.assertIn('X-View', response)
-        self.assertEqual(response['X-View'], 'admin_docs.views.XViewClass')
+        self.assertEqual(response.headers['X-View'], 'admin_docs.views.XViewClass')
         user.is_staff = False
         user.save()
         response = self.client.head('/xview/class/')
@@ -44,4 +45,14 @@ class XViewMiddlewareTest(TestDataMixin, AdminDocsTestCase):
     def test_callable_object_view(self):
         self.client.force_login(self.superuser)
         response = self.client.head('/xview/callable_object/')
-        self.assertEqual(response['X-View'], 'admin_docs.views.XViewCallableObject')
+        self.assertEqual(response.headers['X-View'], 'admin_docs.views.XViewCallableObject')
+
+    @override_settings(MIDDLEWARE=[])
+    def test_no_auth_middleware(self):
+        msg = (
+            "The XView middleware requires authentication middleware to be "
+            "installed. Edit your MIDDLEWARE setting to insert "
+            "'django.contrib.auth.middleware.AuthenticationMiddleware'."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.client.head('/xview/func/')
