@@ -4,6 +4,7 @@ A custom AdminSite for AdminViewPermissionsTest.test_login_has_permission().
 from django.contrib import admin
 from django.contrib.auth import get_permission_codename
 from django.contrib.auth.forms import AuthenticationForm
+from django.core.exceptions import ValidationError
 
 from . import admin as base_admin, models
 
@@ -12,9 +13,8 @@ PERMISSION_NAME = 'admin_views.%s' % get_permission_codename('change', models.Ar
 
 class PermissionAdminAuthenticationForm(AuthenticationForm):
     def confirm_login_allowed(self, user):
-        from django import forms
         if not user.is_active or not (user.is_staff or user.has_perm(PERMISSION_NAME)):
-            raise forms.ValidationError('permission denied')
+            raise ValidationError('permission denied')
 
 
 class HasPermissionAdmin(admin.AdminSite):
