@@ -10,17 +10,11 @@ class Author(models.Model):
     age = models.IntegerField()
     friends = models.ManyToManyField('self', blank=True)
 
-    def __str__(self):
-        return self.name
-
 
 class Publisher(models.Model):
     name = models.CharField(max_length=255)
     num_awards = models.IntegerField()
 
-    def __str__(self):
-        return self.name
-
 
 class ItemTag(models.Model):
     tag = models.CharField(max_length=100)
@@ -44,9 +38,6 @@ class Book(models.Model):
     class Meta:
         ordering = ('name',)
 
-    def __str__(self):
-        return self.name
-
 
 class Store(models.Model):
     name = models.CharField(max_length=255)
@@ -54,9 +45,6 @@ class Store(models.Model):
     original_opening = models.DateTimeField()
     friday_night_closing = models.TimeField()
 
-    def __str__(self):
-        return self.name
-
 
 class Entries(models.Model):
     EntryID = models.AutoField(primary_key=True, db_column='Entry ID')
@@ -81,9 +69,6 @@ class WithManualPK(models.Model):
 class HardbackBook(Book):
     weight = models.FloatField()
 
-    def __str__(self):
-        return "%s (hardback): %s" % (self.name, self.weight)
-
 
 # Models for ticket #21150
 class Alfa(models.Model):
