@@ -1,7 +1,7 @@
-from django.conf.urls import url
 from django.contrib import admin
 from django.contrib.auth import get_user_model
 from django.contrib.auth.admin import UserAdmin
+from django.urls import path
 
 site = admin.AdminSite(name='custom_user_admin')
 
@@ -12,11 +12,12 @@ class CustomUserAdmin(UserAdmin):
         # integer manually to avoid causing an error.
         original_pk = request.user.pk
         request.user.pk = 1
-        super(CustomUserAdmin, self).log_change(request, object, message)
+        super().log_change(request, object, message)
         request.user.pk = original_pk
 
+
 site.register(get_user_model(), CustomUserAdmin)
 
 urlpatterns = [
-    url(r'^admin/', site.urls),
+    path('admin/', site.urls),
 ]
