@@ -1,3 +1,4 @@
+from math import ceil
 from operator import attrgetter
 
 from django.db import IntegrityError, NotSupportedError, connection
@@ -8,9 +9,9 @@ from django.test import (
 )
 
 from .models import (
-    Country, NoFields, NullableFields, Pizzeria, ProxyCountry,
-    ProxyMultiCountry, ProxyMultiProxyCountry, ProxyProxyCountry, Restaurant,
-    State, TwoFields,
+    BigAutoFieldModel, Country, NoFields, NullableFields, Pizzeria,
+    ProxyCountry, ProxyMultiCountry, ProxyMultiProxyCountry, ProxyProxyCountry,
+    Restaurant, SmallAutoFieldModel, State, TwoFields,
 )
 
 
@@ -25,7 +26,7 @@ class BulkCreateTests(TestCase):
 
     def test_simple(self):
         created = Country.objects.bulk_create(self.data)
-        self.assertEqual(len(created), 4)
+        self.assertEqual(created, self.data)
         self.assertQuerysetEqual(Country.objects.order_by("-name"), [
             "United States of America", "The Netherlands", "Germany", "Czech Republic"
         ], attrgetter("name"))
@@ -49,6 +50,16 @@ class BulkCreateTests(TestCase):
         Country.objects.bulk_create([Country(description='Ж' * 3000)])
         self.assertEqual(Country.objects.count(), 1)
 
+    @skipUnlessDBFeature('has_bulk_insert')
+    def test_long_and_short_text(self):
+        Country.objects.bulk_create([
+            Country(description='a' * 4001),
+            Country(description='a'),
+            Country(description='Ж' * 2001),
+            Country(description='Ж'),
+        ])
+        self.assertEqual(Country.objects.count(), 4)
+
     def test_multi_table_inheritance_unsupported(self):
         expected_message = "Can't bulk create a multi-table inherited model"
         with self.assertRaisesMessage(ValueError, expected_message):
@@ -104,7 +115,8 @@ class BulkCreateTests(TestCase):
     def test_zero_as_autoval(self):
         """
         Zero as id for AutoField should raise exception in MySQL, because MySQL
-        does not allow zero for automatic primary key.
+        does not allow zero for automatic primary key if the
+        NO_AUTO_VALUE_ON_ZERO SQL mode is not enabled.
         """
         valid_country = Country(name='Germany', iso_two_letter='DE')
         invalid_country = Country(id=0, name='Poland', iso_two_letter='PL')
@@ -204,6 +216,14 @@ class BulkCreateTests(TestCase):
         with self.assertNumQueries(1):
             TwoFields.objects.bulk_create(objs, len(objs))
 
+    @skipUnlessDBFeature('has_bulk_insert')
+    def test_explicit_batch_size_respects_max_batch_size(self):
+        objs = [Country() for i in range(1000)]
+        fields = ['name', 'iso_two_letter', 'description']
+        max_batch_size = max(connection.ops.bulk_batch_size(fields, objs), 1)
+        with self.assertNumQueries(ceil(len(objs) / max_batch_size)):
+            Country.objects.bulk_create(objs, batch_size=max_batch_size + 1)
+
     @skipUnlessDBFeature('has_bulk_insert')
     def test_bulk_insert_expressions(self):
         Restaurant.objects.bulk_create([
@@ -215,10 +235,16 @@ class BulkCreateTests(TestCase):
 
     @skipUnlessDBFeature('has_bulk_insert')
     def test_bulk_insert_nullable_fields(self):
+        fk_to_auto_fields = {
+            'auto_field': NoFields.objects.create(),
+            'small_auto_field': SmallAutoFieldModel.objects.create(),
+            'big_auto_field': BigAutoFieldModel.objects.create(),
+        }
         # NULL can be mixed with other values in nullable fields
         nullable_fields = [field for field in NullableFields._meta.get_fields() if field.name != 'id']
         NullableFields.objects.bulk_create([
-            NullableFields(**{field.name: None}) for field in nullable_fields
+            NullableFields(**{**fk_to_auto_fields, field.name: None})
+            for field in nullable_fields
         ])
         self.assertEqual(NullableFields.objects.count(), len(nullable_fields))
         for field in nullable_fields:
@@ -226,14 +252,14 @@ class BulkCreateTests(TestCase):
                 field_value = '' if isinstance(field, FileField) else None
                 self.assertEqual(NullableFields.objects.filter(**{field.name: field_value}).count(), 1)
 
-    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    @skipUnlessDBFeature('can_return_rows_from_bulk_insert')
     def test_set_pk_and_insert_single_item(self):
         with self.assertNumQueries(1):
             countries = Country.objects.bulk_create([self.data[0]])
         self.assertEqual(len(countries), 1)
         self.assertEqual(Country.objects.get(pk=countries[0].pk), countries[0])
 
-    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    @skipUnlessDBFeature('can_return_rows_from_bulk_insert')
     def test_set_pk_and_query_efficiency(self):
         with self.assertNumQueries(1):
             countries = Country.objects.bulk_create(self.data)
@@ -243,7 +269,7 @@ class BulkCreateTests(TestCase):
         self.assertEqual(Country.objects.get(pk=countries[2].pk), countries[2])
         self.assertEqual(Country.objects.get(pk=countries[3].pk), countries[3])
 
-    @skipUnlessDBFeature('can_return_ids_from_bulk_insert')
+    @skipUnlessDBFeature('can_return_rows_from_bulk_insert')
     def test_set_state(self):
         country_nl = Country(name='Netherlands', iso_two_letter='NL')
         country_be = Country(name='Belgium', iso_two_letter='BE')
@@ -295,3 +321,29 @@ class BulkCreateTests(TestCase):
         # Without ignore_conflicts=True, there's a problem.
         with self.assertRaises(IntegrityError):
             TwoFields.objects.bulk_create(conflicting_objects)
+
+    def test_nullable_fk_after_parent(self):
+        parent = NoFields()
+        child = NullableFields(auto_field=parent, integer_field=88)
+        parent.save()
+        NullableFields.objects.bulk_create([child])
+        child = NullableFields.objects.get(integer_field=88)
+        self.assertEqual(child.auto_field, parent)
+
+    @skipUnlessDBFeature('can_return_rows_from_bulk_insert')
+    def test_nullable_fk_after_parent_bulk_create(self):
+        parent = NoFields()
+        child = NullableFields(auto_field=parent, integer_field=88)
+        NoFields.objects.bulk_create([parent])
+        NullableFields.objects.bulk_create([child])
+        child = NullableFields.objects.get(integer_field=88)
+        self.assertEqual(child.auto_field, parent)
+
+    def test_unsaved_parent(self):
+        parent = NoFields()
+        msg = (
+            "bulk_create() prohibited to prevent data loss due to unsaved "
+            "related object 'auto_field'."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
+            NullableFields.objects.bulk_create([NullableFields(auto_field=parent)])
