@@ -12,16 +12,10 @@ class Site(models.Model):
     domain = models.CharField(max_length=100)
     objects = SiteManager()
 
-    def __str__(self):
-        return self.domain
-
 
 class Author(models.Model):
     name = models.CharField(max_length=100)
 
-    def __str__(self):
-        return self.name
-
     def get_absolute_url(self):
         return '/authors/%s/' % self.id
 
@@ -32,16 +26,10 @@ class Article(models.Model):
     author = models.ForeignKey(Author, models.CASCADE)
     date_created = models.DateTimeField()
 
-    def __str__(self):
-        return self.title
-
 
 class SchemeIncludedURL(models.Model):
     url = models.URLField(max_length=100)
 
-    def __str__(self):
-        return self.url
-
     def get_absolute_url(self):
         return self.url
 
@@ -62,9 +50,6 @@ class FooWithoutUrl(models.Model):
     """
     name = models.CharField(max_length=30, unique=True)
 
-    def __str__(self):
-        return self.name
-
 
 class FooWithUrl(FooWithoutUrl):
     """
@@ -99,9 +84,6 @@ class Answer(models.Model):
     class Meta:
         order_with_respect_to = 'question'
 
-    def __str__(self):
-        return self.text
-
 
 class Post(models.Model):
     """An ordered tag on an item."""
@@ -114,18 +96,12 @@ class Post(models.Model):
     class Meta:
         order_with_respect_to = 'parent'
 
-    def __str__(self):
-        return self.title
-
 
 class ModelWithNullFKToSite(models.Model):
     title = models.CharField(max_length=200)
     site = models.ForeignKey(Site, null=True, on_delete=models.CASCADE)
     post = models.ForeignKey(Post, null=True, on_delete=models.CASCADE)
 
-    def __str__(self):
-        return self.title
-
     def get_absolute_url(self):
         return '/title/%s/' % quote(self.title)
 
