@@ -201,7 +201,15 @@ class ContentTypesTests(TestCase):
 
     def test_str(self):
         ct = ContentType.objects.get(app_label='contenttypes_tests', model='site')
-        self.assertEqual(str(ct), 'site')
+        self.assertEqual(str(ct), 'contenttypes_tests | site')
+
+    def test_app_labeled_name(self):
+        ct = ContentType.objects.get(app_label='contenttypes_tests', model='site')
+        self.assertEqual(ct.app_labeled_name, 'contenttypes_tests | site')
+
+    def test_app_labeled_name_unknown_model(self):
+        ct = ContentType(app_label='contenttypes_tests', model='unknown')
+        self.assertEqual(ct.app_labeled_name, 'unknown')
 
 
 class TestRouter:
