@@ -6,10 +6,15 @@ import uuid
 from django.db import models
 
 
+class Manager(models.Model):
+    name = models.CharField(max_length=50)
+
+
 class Employee(models.Model):
     firstname = models.CharField(max_length=50)
     lastname = models.CharField(max_length=50)
     salary = models.IntegerField(blank=True, null=True)
+    manager = models.ForeignKey(Manager, models.CASCADE, null=True)
 
     def __str__(self):
         return '%s %s' % (self.firstname, self.lastname)
@@ -34,6 +39,7 @@ class Company(models.Model):
         related_name='company_point_of_contact_set',
         null=True,
     )
+    based_in_eu = models.BooleanField(default=False)
 
     def __str__(self):
         return self.name
@@ -42,9 +48,10 @@ class Company(models.Model):
 class Number(models.Model):
     integer = models.BigIntegerField(db_column='the_integer')
     float = models.FloatField(null=True, db_column='the_float')
+    decimal_value = models.DecimalField(max_digits=20, decimal_places=17, null=True)
 
     def __str__(self):
-        return '%i, %.3f' % (self.integer, self.float)
+        return '%i, %.3f, %.17f' % (self.integer, self.float, self.decimal_value)
 
 
 class Experiment(models.Model):
@@ -75,7 +82,7 @@ class Time(models.Model):
     time = models.TimeField(null=True)
 
     def __str__(self):
-        return "%s" % self.time
+        return str(self.time)
 
 
 class SimulationRun(models.Model):
@@ -94,6 +101,3 @@ class UUIDPK(models.Model):
 class UUID(models.Model):
     uuid = models.UUIDField(null=True)
     uuid_fk = models.ForeignKey(UUIDPK, models.CASCADE, null=True)
-
-    def __str__(self):
-        return "%s" % self.uuid
