@@ -1,7 +1,6 @@
 from datetime import datetime
 
 from django.test import TestCase
-from django.utils import six
 
 from .models import Article
 
@@ -12,6 +11,6 @@ class DefaultTests(TestCase):
         now = datetime.now()
         a.save()
 
-        self.assertIsInstance(a.id, six.integer_types)
+        self.assertIsInstance(a.id, int)
         self.assertEqual(a.headline, "Default headline")
-        self.assertTrue((now - a.pub_date).seconds < 5)
+        self.assertLess((now - a.pub_date).seconds, 5)
