@@ -6,13 +6,14 @@ import sys
 import tempfile as sys_tempfile
 import unittest
 from io import BytesIO, StringIO
+from unittest import mock
 from urllib.parse import quote
 
 from django.core.exceptions import SuspiciousFileOperation
 from django.core.files import temp as tempfile
 from django.core.files.uploadedfile import SimpleUploadedFile, UploadedFile
 from django.http.multipartparser import (
-    MultiPartParser, MultiPartParserError, parse_header,
+    FILE, MultiPartParser, MultiPartParserError, Parser, parse_header,
 )
 from django.test import SimpleTestCase, TestCase, client, override_settings
 
@@ -36,6 +37,7 @@ CANDIDATE_TRAVERSAL_FILE_NAMES = [
     '..\\..\\hax0rd.txt',       # Relative path, win-style.
     '../..\\hax0rd.txt',        # Relative path, mixed.
     '..&#x2F;hax0rd.txt',       # HTML entities.
+    '..&sol;hax0rd.txt',        # HTML entities.
 ]
 
 CANDIDATE_INVALID_FILE_NAMES = [
@@ -55,8 +57,7 @@ class FileUploadTests(TestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        if not os.path.isdir(MEDIA_ROOT):
-            os.makedirs(MEDIA_ROOT)
+        os.makedirs(MEDIA_ROOT, exist_ok=True)
 
     @classmethod
     def tearDownClass(cls):
@@ -225,15 +226,61 @@ class FileUploadTests(TestCase):
         response = self.client.request(**r)
         self.assertEqual(response.status_code, 200)
 
+    def test_unicode_file_name_rfc2231_with_double_quotes(self):
+        payload = client.FakePayload()
+        payload.write('\r\n'.join([
+            '--' + client.BOUNDARY,
+            'Content-Disposition: form-data; name="file_unicode"; '
+            'filename*="UTF-8\'\'%s"' % quote(UNICODE_FILENAME),
+            'Content-Type: application/octet-stream',
+            '',
+            'You got pwnd.\r\n',
+            '\r\n--' + client.BOUNDARY + '--\r\n',
+        ]))
+        r = {
+            'CONTENT_LENGTH': len(payload),
+            'CONTENT_TYPE': client.MULTIPART_CONTENT,
+            'PATH_INFO': '/unicode_name/',
+            'REQUEST_METHOD': 'POST',
+            'wsgi.input': payload,
+        }
+        response = self.client.request(**r)
+        self.assertEqual(response.status_code, 200)
+
+    def test_unicode_name_rfc2231_with_double_quotes(self):
+        payload = client.FakePayload()
+        payload.write('\r\n'.join([
+            '--' + client.BOUNDARY,
+            'Content-Disposition: form-data; name*="UTF-8\'\'file_unicode"; '
+            'filename*="UTF-8\'\'%s"' % quote(UNICODE_FILENAME),
+            'Content-Type: application/octet-stream',
+            '',
+            'You got pwnd.\r\n',
+            '\r\n--' + client.BOUNDARY + '--\r\n'
+        ]))
+        r = {
+            'CONTENT_LENGTH': len(payload),
+            'CONTENT_TYPE': client.MULTIPART_CONTENT,
+            'PATH_INFO': '/unicode_name/',
+            'REQUEST_METHOD': 'POST',
+            'wsgi.input': payload,
+        }
+        response = self.client.request(**r)
+        self.assertEqual(response.status_code, 200)
+
     def test_blank_filenames(self):
         """
         Receiving file upload when filename is blank (before and after
         sanitization) should be okay.
         """
-        # The second value is normalized to an empty name by
-        # MultiPartParser.IE_sanitize()
-        filenames = ['', 'C:\\Windows\\']
-
+        filenames = [
+            '',
+            # Normalized by MultiPartParser.IE_sanitize().
+            'C:\\Windows\\',
+            # Normalized by os.path.basename().
+            '/',
+            'ends-with-slash/',
+        ]
         payload = client.FakePayload()
         for i, name in enumerate(filenames):
             payload.write('\r\n'.join([
@@ -438,6 +485,38 @@ class FileUploadTests(TestCase):
             with self.assertRaisesMessage(AttributeError, msg):
                 self.client.post('/quota/broken/', {'f': file})
 
+    def test_stop_upload_temporary_file_handler(self):
+        with tempfile.NamedTemporaryFile() as temp_file:
+            temp_file.write(b'a')
+            temp_file.seek(0)
+            response = self.client.post('/temp_file/stop_upload/', {'file': temp_file})
+            temp_path = response.json()['temp_path']
+            self.assertIs(os.path.exists(temp_path), False)
+
+    def test_upload_interrupted_temporary_file_handler(self):
+        # Simulate an interrupted upload by omitting the closing boundary.
+        class MockedParser(Parser):
+            def __iter__(self):
+                for item in super().__iter__():
+                    item_type, meta_data, field_stream = item
+                    yield item_type, meta_data, field_stream
+                    if item_type == FILE:
+                        return
+
+        with tempfile.NamedTemporaryFile() as temp_file:
+            temp_file.write(b'a')
+            temp_file.seek(0)
+            with mock.patch(
+                'django.http.multipartparser.Parser',
+                MockedParser,
+            ):
+                response = self.client.post(
+                    '/temp_file/upload_interrupted/',
+                    {'file': temp_file},
+                )
+            temp_path = response.json()['temp_path']
+            self.assertIs(os.path.exists(temp_path), False)
+
     def test_fileupload_getlist(self):
         file = tempfile.NamedTemporaryFile
         with file() as file1, file() as file2, file() as file2a:
@@ -528,8 +607,9 @@ class FileUploadTests(TestCase):
             try:
                 self.client.post('/upload_errors/', post_data)
             except reference_error.__class__ as err:
-                self.assertFalse(
-                    str(err) == str(reference_error),
+                self.assertNotEqual(
+                    str(err),
+                    str(reference_error),
                     "Caught a repeated exception that'll cause an infinite loop in file uploads."
                 )
             except Exception as err:
@@ -569,32 +649,43 @@ class FileUploadTests(TestCase):
     def test_filename_traversal_upload(self):
         os.makedirs(UPLOAD_TO, exist_ok=True)
         self.addCleanup(shutil.rmtree, MEDIA_ROOT)
-        file_name = '..&#x2F;test.txt',
-        payload = client.FakePayload()
-        payload.write(
-            '\r\n'.join([
-                '--' + client.BOUNDARY,
-                'Content-Disposition: form-data; name="my_file"; '
-                'filename="%s";' % file_name,
-                'Content-Type: text/plain',
-                '',
-                'file contents.\r\n',
-                '\r\n--' + client.BOUNDARY + '--\r\n',
-            ]),
-        )
-        r = {
-            'CONTENT_LENGTH': len(payload),
-            'CONTENT_TYPE': client.MULTIPART_CONTENT,
-            'PATH_INFO': '/upload_traversal/',
-            'REQUEST_METHOD': 'POST',
-            'wsgi.input': payload,
-        }
-        response = self.client.request(**r)
-        result = response.json()
-        self.assertEqual(response.status_code, 200)
-        self.assertEqual(result['file_name'], 'test.txt')
-        self.assertIs(os.path.exists(os.path.join(MEDIA_ROOT, 'test.txt')), False)
-        self.assertIs(os.path.exists(os.path.join(UPLOAD_TO, 'test.txt')), True)
+        tests = [
+            '..&#x2F;test.txt',
+            '..&sol;test.txt',
+        ]
+        for file_name in tests:
+            with self.subTest(file_name=file_name):
+                payload = client.FakePayload()
+                payload.write(
+                    '\r\n'.join([
+                        '--' + client.BOUNDARY,
+                        'Content-Disposition: form-data; name="my_file"; '
+                        'filename="%s";' % file_name,
+                        'Content-Type: text/plain',
+                        '',
+                        'file contents.\r\n',
+                        '\r\n--' + client.BOUNDARY + '--\r\n',
+                    ]),
+                )
+                r = {
+                    'CONTENT_LENGTH': len(payload),
+                    'CONTENT_TYPE': client.MULTIPART_CONTENT,
+                    'PATH_INFO': '/upload_traversal/',
+                    'REQUEST_METHOD': 'POST',
+                    'wsgi.input': payload,
+                }
+                response = self.client.request(**r)
+                result = response.json()
+                self.assertEqual(response.status_code, 200)
+                self.assertEqual(result['file_name'], 'test.txt')
+                self.assertIs(
+                    os.path.exists(os.path.join(MEDIA_ROOT, 'test.txt')),
+                    False,
+                )
+                self.assertIs(
+                    os.path.exists(os.path.join(UPLOAD_TO, 'test.txt')),
+                    True,
+                )
 
 
 @override_settings(MEDIA_ROOT=MEDIA_ROOT)
@@ -606,8 +697,7 @@ class DirectoryCreationTests(SimpleTestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        if not os.path.isdir(MEDIA_ROOT):
-            os.makedirs(MEDIA_ROOT)
+        os.makedirs(MEDIA_ROOT, exist_ok=True)
 
     @classmethod
     def tearDownClass(cls):
@@ -626,16 +716,13 @@ class DirectoryCreationTests(SimpleTestCase):
             self.obj.testfile.save('foo.txt', SimpleUploadedFile('foo.txt', b'x'), save=False)
 
     def test_not_a_directory(self):
-        """The correct IOError is raised when the upload directory name exists but isn't a directory"""
         # Create a file with the upload directory name
         open(UPLOAD_TO, 'wb').close()
         self.addCleanup(os.remove, UPLOAD_TO)
-        with self.assertRaises(IOError) as exc_info:
+        msg = '%s exists and is not a directory.' % UPLOAD_TO
+        with self.assertRaisesMessage(FileExistsError, msg):
             with SimpleUploadedFile('foo.txt', b'x') as file:
                 self.obj.testfile.save('foo.txt', file, save=False)
-        # The test needs to be done on a specific string as IOError
-        # is raised even without the patch (just not early enough)
-        self.assertEqual(exc_info.exception.args[0], "%s exists and is not a directory." % UPLOAD_TO)
 
 
 class MultiParserTests(SimpleTestCase):
