@@ -2,4 +2,4 @@ from django.apps import AppConfig
 
 
 class LoadingAppConfig(AppConfig):
-    name = 'loading_app'
+    name = 'i18n.loading_app'
