@@ -1,3 +1,4 @@
+import os
 import re
 from io import StringIO
 from unittest import mock, skipUnless
@@ -7,7 +8,7 @@ from django.db import connection
 from django.db.backends.base.introspection import TableInfo
 from django.test import TestCase, TransactionTestCase, skipUnlessDBFeature
 
-from .models import PeopleMoreData
+from .models import PeopleMoreData, test_collation
 
 
 def inspectdb_tables_only(table_name):
@@ -18,6 +19,13 @@ def inspectdb_tables_only(table_name):
     return table_name.startswith('inspectdb_')
 
 
+def inspectdb_views_only(table_name):
+    return (
+        table_name.startswith('inspectdb_') and
+        table_name.endswith(('_materialized', '_view'))
+    )
+
+
 def special_table_only(table_name):
     return table_name.startswith('inspectdb_special')
 
@@ -52,7 +60,7 @@ class InspectDBTestCase(TestCase):
         output = out.getvalue()
 
         def assertFieldType(name, definition):
-            out_def = re.search(r'^\s*%s = (models.*)$' % name, output, re.MULTILINE).groups()[0]
+            out_def = re.search(r'^\s*%s = (models.*)$' % name, output, re.MULTILINE)[1]
             self.assertEqual(definition, out_def)
 
         return assertFieldType
@@ -60,10 +68,11 @@ class InspectDBTestCase(TestCase):
     def test_field_types(self):
         """Test introspection of various Django field types"""
         assertFieldType = self.make_field_type_asserter()
-
+        introspected_field_types = connection.features.introspected_field_types
+        char_field_type = introspected_field_types['CharField']
         # Inspecting Oracle DB doesn't produce correct results (#19884):
         # - it reports fields as blank=True when they aren't.
-        if not connection.features.interprets_empty_strings_as_nulls:
+        if not connection.features.interprets_empty_strings_as_nulls and char_field_type == 'CharField':
             assertFieldType('char_field', "models.CharField(max_length=10)")
             assertFieldType('null_char_field', "models.CharField(max_length=10, blank=True, null=True)")
             assertFieldType('email_field', "models.CharField(max_length=254)")
@@ -72,36 +81,89 @@ class InspectDBTestCase(TestCase):
             assertFieldType('slug_field', "models.CharField(max_length=50)")
             assertFieldType('text_field', "models.TextField()")
             assertFieldType('url_field', "models.CharField(max_length=200)")
+        if char_field_type == 'TextField':
+            assertFieldType('char_field', 'models.TextField()')
+            assertFieldType('null_char_field', 'models.TextField(blank=True, null=True)')
+            assertFieldType('email_field', 'models.TextField()')
+            assertFieldType('file_field', 'models.TextField()')
+            assertFieldType('file_path_field', 'models.TextField()')
+            assertFieldType('slug_field', 'models.TextField()')
+            assertFieldType('text_field', 'models.TextField()')
+            assertFieldType('url_field', 'models.TextField()')
         assertFieldType('date_field', "models.DateField()")
         assertFieldType('date_time_field', "models.DateTimeField()")
-        if connection.features.can_introspect_ip_address_field:
+        if introspected_field_types['GenericIPAddressField'] == 'GenericIPAddressField':
             assertFieldType('gen_ip_address_field', "models.GenericIPAddressField()")
         elif not connection.features.interprets_empty_strings_as_nulls:
             assertFieldType('gen_ip_address_field', "models.CharField(max_length=39)")
-        if connection.features.can_introspect_time_field:
-            assertFieldType('time_field', "models.TimeField()")
+        assertFieldType('time_field', 'models.%s()' % introspected_field_types['TimeField'])
         if connection.features.has_native_uuid_field:
             assertFieldType('uuid_field', "models.UUIDField()")
         elif not connection.features.interprets_empty_strings_as_nulls:
             assertFieldType('uuid_field', "models.CharField(max_length=32)")
 
+    @skipUnlessDBFeature('can_introspect_json_field', 'supports_json_field')
+    def test_json_field(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_jsonfieldcolumntype', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn('json_field = models.JSONField()', output)
+        self.assertIn('null_json_field = models.JSONField(blank=True, null=True)', output)
+
+    @skipUnlessDBFeature('supports_collation_on_charfield')
+    @skipUnless(test_collation, 'Language collations are not supported.')
+    def test_char_field_db_collation(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_charfielddbcollation', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn(
+                "char_field = models.CharField(max_length=10, "
+                "db_collation='%s')" % test_collation,
+                output,
+            )
+        else:
+            self.assertIn(
+                "char_field = models.CharField(max_length=10, "
+                "db_collation='%s', blank=True, null=True)" % test_collation,
+                output,
+            )
+
+    @skipUnlessDBFeature('supports_collation_on_textfield')
+    @skipUnless(test_collation, 'Language collations are not supported.')
+    def test_text_field_db_collation(self):
+        out = StringIO()
+        call_command('inspectdb', 'inspectdb_textfielddbcollation', stdout=out)
+        output = out.getvalue()
+        if not connection.features.interprets_empty_strings_as_nulls:
+            self.assertIn(
+                "text_field = models.TextField(db_collation='%s')" % test_collation,
+                output,
+            )
+        else:
+            self.assertIn(
+                "text_field = models.TextField(db_collation='%s, blank=True, "
+                "null=True)" % test_collation,
+                output,
+            )
+
     def test_number_field_types(self):
         """Test introspection of various Django field types"""
         assertFieldType = self.make_field_type_asserter()
+        introspected_field_types = connection.features.introspected_field_types
 
-        if not connection.features.can_introspect_autofield:
-            assertFieldType('id', "models.IntegerField(primary_key=True)  # AutoField?")
+        auto_field_type = connection.features.introspected_field_types['AutoField']
+        if auto_field_type != 'AutoField':
+            assertFieldType('id', "models.%s(primary_key=True)  # AutoField?" % auto_field_type)
 
-        if connection.features.can_introspect_big_integer_field:
-            assertFieldType('big_int_field', "models.BigIntegerField()")
-        else:
-            assertFieldType('big_int_field', "models.IntegerField()")
+        assertFieldType('big_int_field', 'models.%s()' % introspected_field_types['BigIntegerField'])
 
-        bool_field_type = connection.features.introspected_boolean_field_type
+        bool_field_type = introspected_field_types['BooleanField']
         assertFieldType('bool_field', "models.{}()".format(bool_field_type))
         assertFieldType('null_bool_field', 'models.{}(blank=True, null=True)'.format(bool_field_type))
 
-        if connection.features.can_introspect_decimal_field:
+        if connection.vendor != 'sqlite':
             assertFieldType('decimal_field', "models.DecimalField(max_digits=6, decimal_places=1)")
         else:       # Guessed arguments on SQLite, see #5014
             assertFieldType('decimal_field', "models.DecimalField(max_digits=10, decimal_places=5)  "
@@ -109,29 +171,11 @@ class InspectDBTestCase(TestCase):
                                              "as this database handles decimal fields as float")
 
         assertFieldType('float_field', "models.FloatField()")
-
-        assertFieldType('int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_positive_integer_field:
-            assertFieldType('pos_int_field', "models.PositiveIntegerField()")
-        else:
-            assertFieldType('pos_int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_positive_integer_field:
-            if connection.features.can_introspect_small_integer_field:
-                assertFieldType('pos_small_int_field', "models.PositiveSmallIntegerField()")
-            else:
-                assertFieldType('pos_small_int_field', "models.PositiveIntegerField()")
-        else:
-            if connection.features.can_introspect_small_integer_field:
-                assertFieldType('pos_small_int_field', "models.SmallIntegerField()")
-            else:
-                assertFieldType('pos_small_int_field', "models.IntegerField()")
-
-        if connection.features.can_introspect_small_integer_field:
-            assertFieldType('small_int_field', "models.SmallIntegerField()")
-        else:
-            assertFieldType('small_int_field', "models.IntegerField()")
+        assertFieldType('int_field', 'models.%s()' % introspected_field_types['IntegerField'])
+        assertFieldType('pos_int_field', 'models.%s()' % introspected_field_types['PositiveIntegerField'])
+        assertFieldType('pos_big_int_field', 'models.%s()' % introspected_field_types['PositiveBigIntegerField'])
+        assertFieldType('pos_small_int_field', 'models.%s()' % introspected_field_types['PositiveSmallIntegerField'])
+        assertFieldType('small_int_field', 'models.%s()' % introspected_field_types['SmallIntegerField'])
 
     @skipUnlessDBFeature('can_introspect_foreign_keys')
     def test_attribute_name_not_python_keyword(self):
@@ -152,29 +196,30 @@ class InspectDBTestCase(TestCase):
             output,
         )
         self.assertIn(
-            "people_pk = models.ForeignKey(InspectdbPeople, models.DO_NOTHING, primary_key=True)",
+            'people_pk = models.OneToOneField(InspectdbPeople, models.DO_NOTHING, primary_key=True)',
             output,
         )
         self.assertIn(
-            "people_unique = models.ForeignKey(InspectdbPeople, models.DO_NOTHING, unique=True)",
+            'people_unique = models.OneToOneField(InspectdbPeople, models.DO_NOTHING)',
             output,
         )
 
     def test_digits_column_name_introspection(self):
         """Introspection of column names consist/start with digits (#16536/#17676)"""
+        char_field_type = connection.features.introspected_field_types['CharField']
         out = StringIO()
         call_command('inspectdb', 'inspectdb_digitsincolumnname', stdout=out)
         output = out.getvalue()
         error_message = "inspectdb generated a model field name which is a number"
-        self.assertNotIn("    123 = models.CharField", output, msg=error_message)
-        self.assertIn("number_123 = models.CharField", output)
+        self.assertNotIn('    123 = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_123 = models.%s' % char_field_type, output)
 
         error_message = "inspectdb generated a model field name which starts with a digit"
-        self.assertNotIn("    4extra = models.CharField", output, msg=error_message)
-        self.assertIn("number_4extra = models.CharField", output)
+        self.assertNotIn('    4extra = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_4extra = models.%s' % char_field_type, output)
 
-        self.assertNotIn("    45extra = models.CharField", output, msg=error_message)
-        self.assertIn("number_45extra = models.CharField", output)
+        self.assertNotIn('    45extra = models.%s' % char_field_type, output, msg=error_message)
+        self.assertIn('number_45extra = models.%s' % char_field_type, output)
 
     def test_special_column_name_introspection(self):
         """
@@ -185,12 +230,13 @@ class InspectDBTestCase(TestCase):
         call_command('inspectdb', table_name_filter=special_table_only, stdout=out)
         output = out.getvalue()
         base_name = connection.introspection.identifier_converter('Field')
-        self.assertIn("field = models.IntegerField()", output)
-        self.assertIn("field_field = models.IntegerField(db_column='%s_')" % base_name, output)
-        self.assertIn("field_field_0 = models.IntegerField(db_column='%s__')" % base_name, output)
-        self.assertIn("field_field_1 = models.IntegerField(db_column='__field')", output)
-        self.assertIn("prc_x = models.IntegerField(db_column='prc(%) x')", output)
-        self.assertIn("tamaño = models.IntegerField()", output)
+        integer_field_type = connection.features.introspected_field_types['IntegerField']
+        self.assertIn("field = models.%s()" % integer_field_type, output)
+        self.assertIn("field_field = models.%s(db_column='%s_')" % (integer_field_type, base_name), output)
+        self.assertIn("field_field_0 = models.%s(db_column='%s__')" % (integer_field_type, base_name), output)
+        self.assertIn("field_field_1 = models.%s(db_column='__field')" % integer_field_type, output)
+        self.assertIn("prc_x = models.{}(db_column='prc(%) x')".format(integer_field_type), output)
+        self.assertIn("tamaño = models.%s()" % integer_field_type, output)
 
     def test_table_name_introspection(self):
         """
@@ -215,7 +261,7 @@ class InspectDBTestCase(TestCase):
         call_command('inspectdb', 'inspectdb_uniquetogether', stdout=out)
         output = out.getvalue()
         self.assertIn("    unique_together = (('", output)
-        unique_together_match = re.findall(self.unique_re, output)
+        unique_together_match = self.unique_re.findall(output)
         # There should be one unique_together tuple.
         self.assertEqual(len(unique_together_match), 1)
         fields = unique_together_match[0]
@@ -244,7 +290,7 @@ class InspectDBTestCase(TestCase):
             )
             output = out.getvalue()
             self.assertIn('# A unique constraint could not be introspected.', output)
-            self.assertEqual(re.findall(self.unique_re, output), ["('id', 'people_unique')"])
+            self.assertEqual(self.unique_re.findall(output), ["('id', 'people_unique')"])
         finally:
             with connection.cursor() as c:
                 c.execute('DROP INDEX Findex')
@@ -256,18 +302,17 @@ class InspectDBTestCase(TestCase):
         Introspection of columns with a custom field (#21090)
         """
         out = StringIO()
-        orig_data_types_reverse = connection.introspection.data_types_reverse
-        try:
-            connection.introspection.data_types_reverse = {
+        with mock.patch(
+            'django.db.connection.introspection.data_types_reverse.base_data_types_reverse',
+            {
                 'text': 'myfields.TextField',
                 'bigint': 'BigIntegerField',
-            }
+            },
+        ):
             call_command('inspectdb', 'inspectdb_columntypes', stdout=out)
             output = out.getvalue()
             self.assertIn("text_field = myfields.TextField()", output)
             self.assertIn("big_int_field = models.BigIntegerField()", output)
-        finally:
-            connection.introspection.data_types_reverse = orig_data_types_reverse
 
     def test_introspection_errors(self):
         """
@@ -298,11 +343,20 @@ class InspectDBTransactionalTests(TransactionTestCase):
         view_model = 'class InspectdbPeopleView(models.Model):'
         view_managed = 'managed = False  # Created from a view.'
         try:
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                stdout=out,
+            )
             no_views_output = out.getvalue()
             self.assertNotIn(view_model, no_views_output)
             self.assertNotIn(view_managed, no_views_output)
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, include_views=True, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                include_views=True,
+                stdout=out,
+            )
             with_views_output = out.getvalue()
             self.assertIn(view_model, with_views_output)
             self.assertIn(view_managed, with_views_output)
@@ -322,11 +376,20 @@ class InspectDBTransactionalTests(TransactionTestCase):
         view_model = 'class InspectdbPeopleMaterialized(models.Model):'
         view_managed = 'managed = False  # Created from a view.'
         try:
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                stdout=out,
+            )
             no_views_output = out.getvalue()
             self.assertNotIn(view_model, no_views_output)
             self.assertNotIn(view_managed, no_views_output)
-            call_command('inspectdb', table_name_filter=inspectdb_tables_only, include_views=True, stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_views_only,
+                include_views=True,
+                stdout=out,
+            )
             with_views_output = out.getvalue()
             self.assertIn(view_model, with_views_output)
             self.assertIn(view_managed, with_views_output)
@@ -380,14 +443,18 @@ class InspectDBTransactionalTests(TransactionTestCase):
                     sepal_length real,
                     sepal_width real
                 ) SERVER inspectdb_server OPTIONS (
-                    filename '/dev/null'
+                    filename %s
                 )
-            ''')
+            ''', [os.devnull])
         out = StringIO()
         foreign_table_model = 'class InspectdbIrisForeignTable(models.Model):'
         foreign_table_managed = 'managed = False'
         try:
-            call_command('inspectdb', stdout=out)
+            call_command(
+                'inspectdb',
+                table_name_filter=inspectdb_tables_only,
+                stdout=out,
+            )
             output = out.getvalue()
             self.assertIn(foreign_table_model, output)
             self.assertIn(foreign_table_managed, output)
