@@ -13,22 +13,19 @@ class TimeFieldLookupTests(TestCase):
         self.al3 = Alarm.objects.create(desc='Precise', time='12:34:56')
 
     def test_hour_lookups(self):
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Alarm.objects.filter(time__hour=5),
-            ['<Alarm: 05:30:00 (Early)>'],
-            ordered=False
+            [self.al1],
         )
 
     def test_minute_lookups(self):
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Alarm.objects.filter(time__minute=30),
-            ['<Alarm: 05:30:00 (Early)>'],
-            ordered=False
+            [self.al1],
         )
 
     def test_second_lookups(self):
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Alarm.objects.filter(time__second=56),
-            ['<Alarm: 12:34:56 (Precise)>'],
-            ordered=False
+            [self.al3],
         )
