@@ -10,8 +10,9 @@ class Person(models.Model):
     class Meta:
         ordering = ('name',)
 
-    def __str__(self):
-        return self.name
+
+class PersonChild(Person):
+    pass
 
 
 class Group(models.Model):
@@ -27,9 +28,6 @@ class Group(models.Model):
     class Meta:
         ordering = ('name',)
 
-    def __str__(self):
-        return self.name
-
 
 class Membership(models.Model):
     person = models.ForeignKey(Person, models.CASCADE)
@@ -72,9 +70,7 @@ class TestNoDefaultsOrNulls(models.Model):
 class PersonSelfRefM2M(models.Model):
     name = models.CharField(max_length=5)
     friends = models.ManyToManyField('self', through="Friendship", symmetrical=False)
-
-    def __str__(self):
-        return self.name
+    sym_friends = models.ManyToManyField('self', through='SymmetricalFriendship', symmetrical=True)
 
 
 class Friendship(models.Model):
@@ -83,18 +79,22 @@ class Friendship(models.Model):
     date_friended = models.DateTimeField()
 
 
+class SymmetricalFriendship(models.Model):
+    first = models.ForeignKey(PersonSelfRefM2M, models.CASCADE)
+    second = models.ForeignKey(PersonSelfRefM2M, models.CASCADE, related_name='+')
+    date_friended = models.DateField()
+
+
 # Custom through link fields
 class Event(models.Model):
     title = models.CharField(max_length=50)
     invitees = models.ManyToManyField(
-        Person, through='Invitation',
-        through_fields=('event', 'invitee'),
+        to=Person,
+        through='Invitation',
+        through_fields=['event', 'invitee'],
         related_name='events_invited',
     )
 
-    def __str__(self):
-        return self.title
-
 
 class Invitation(models.Model):
     event = models.ForeignKey(Event, models.CASCADE, related_name='invitations')
@@ -115,9 +115,6 @@ class Employee(models.Model):
     class Meta:
         ordering = ('pk',)
 
-    def __str__(self):
-        return self.name
-
 
 class Relationship(models.Model):
     # field order is deliberately inverted.
