@@ -1,6 +1,7 @@
 from django.contrib.messages import constants
 from django.contrib.messages.storage.base import Message
 from django.contrib.messages.storage.session import SessionStorage
+from django.http import HttpRequest
 from django.test import TestCase
 from django.utils.safestring import SafeData, mark_safe
 
@@ -34,6 +35,15 @@ class SessionTests(BaseTests, TestCase):
     def stored_messages_count(self, storage, response):
         return stored_session_messages_count(storage)
 
+    def test_no_session(self):
+        msg = (
+            'The session-based temporary message storage requires session '
+            'middleware to be installed, and come before the message '
+            'middleware in the MIDDLEWARE list.'
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.storage_class(HttpRequest())
+
     def test_get(self):
         storage = self.storage_class(self.get_request())
         example_messages = ['test', 'me']
