@@ -1,6 +1,7 @@
 from django.db import migrations, models
 from django.db.migrations import operations
 from django.db.migrations.optimizer import MigrationOptimizer
+from django.db.migrations.serializer import serializer_factory
 from django.test import SimpleTestCase
 
 from .models import EmptyManager, UnicodeModel
@@ -18,10 +19,13 @@ class OptimizerTests(SimpleTestCase):
         optimizer = MigrationOptimizer()
         return optimizer.optimize(operations, app_label), optimizer._iterations
 
+    def serialize(self, value):
+        return serializer_factory(value).serialize()[0]
+
     def assertOptimizesTo(self, operations, expected, exact=None, less_than=None, app_label=None):
-        result, iterations = self.optimize(operations, app_label)
-        result = [repr(f.deconstruct()) for f in result]
-        expected = [repr(f.deconstruct()) for f in expected]
+        result, iterations = self.optimize(operations, app_label or 'migrations')
+        result = [self.serialize(f) for f in result]
+        expected = [self.serialize(f) for f in expected]
         self.assertEqual(expected, result)
         if exact is not None and iterations != exact:
             raise self.failureException(
@@ -35,6 +39,11 @@ class OptimizerTests(SimpleTestCase):
     def assertDoesNotOptimize(self, operations, **kwargs):
         self.assertOptimizesTo(operations, operations, **kwargs)
 
+    def test_none_app_label(self):
+        optimizer = MigrationOptimizer()
+        with self.assertRaisesMessage(TypeError, 'app_label must be a str'):
+            optimizer.optimize([], None)
+
     def test_single(self):
         """
         The optimizer does nothing on a single operation,
@@ -110,6 +119,42 @@ class OptimizerTests(SimpleTestCase):
             ]
         )
 
+    def test_create_model_and_remove_model_options(self):
+        self.assertOptimizesTo(
+            [
+                migrations.CreateModel(
+                    'MyModel',
+                    fields=[],
+                    options={'verbose_name': 'My Model'},
+                ),
+                migrations.AlterModelOptions('MyModel', options={}),
+            ],
+            [migrations.CreateModel('MyModel', fields=[])],
+        )
+        self.assertOptimizesTo(
+            [
+                migrations.CreateModel(
+                    'MyModel',
+                    fields=[],
+                    options={
+                        'verbose_name': 'My Model',
+                        'verbose_name_plural': 'My Model plural',
+                    },
+                ),
+                migrations.AlterModelOptions(
+                    'MyModel',
+                    options={'verbose_name': 'My Model'},
+                ),
+            ],
+            [
+                migrations.CreateModel(
+                    'MyModel',
+                    fields=[],
+                    options={'verbose_name': 'My Model'},
+                ),
+            ],
+        )
+
     def _test_create_alter_foo_delete_model(self, alter_foo):
         """
         CreateModel, AlterModelTable, AlterUniqueTogether/AlterIndexTogether/
@@ -208,16 +253,8 @@ class OptimizerTests(SimpleTestCase):
             ],
             [],
         )
-        # This should not work - FK should block it
-        self.assertDoesNotOptimize(
-            [
-                migrations.CreateModel("Foo", [("name", models.CharField(max_length=255))]),
-                migrations.CreateModel("Bar", [("other", models.ForeignKey("testapp.Foo", models.CASCADE))]),
-                migrations.DeleteModel("Foo"),
-            ],
-        )
-        # The same operations should be optimized if app_label is specified and
-        # a FK references a model from the other app.
+        # Operations should be optimized if the FK references a model from the
+        # other app.
         self.assertOptimizesTo(
             [
                 migrations.CreateModel("Foo", [("name", models.CharField(max_length=255))]),
@@ -231,6 +268,13 @@ class OptimizerTests(SimpleTestCase):
         )
         # But it shouldn't work if a FK references a model with the same
         # app_label.
+        self.assertDoesNotOptimize(
+            [
+                migrations.CreateModel('Foo', [('name', models.CharField(max_length=255))]),
+                migrations.CreateModel('Bar', [('other', models.ForeignKey('Foo', models.CASCADE))]),
+                migrations.DeleteModel('Foo'),
+            ],
+        )
         self.assertDoesNotOptimize(
             [
                 migrations.CreateModel("Foo", [("name", models.CharField(max_length=255))]),
@@ -240,12 +284,20 @@ class OptimizerTests(SimpleTestCase):
             app_label="testapp",
         )
         # This should not work - bases should block it
+        self.assertDoesNotOptimize(
+            [
+                migrations.CreateModel('Foo', [('name', models.CharField(max_length=255))]),
+                migrations.CreateModel('Bar', [('size', models.IntegerField())], bases=('Foo',)),
+                migrations.DeleteModel('Foo'),
+            ],
+        )
         self.assertDoesNotOptimize(
             [
                 migrations.CreateModel("Foo", [("name", models.CharField(max_length=255))]),
                 migrations.CreateModel("Bar", [("size", models.IntegerField())], bases=("testapp.Foo",)),
                 migrations.DeleteModel("Foo"),
             ],
+            app_label='testapp',
         )
         # The same operations should be optimized if app_label and none of
         # bases belong to that app.
@@ -289,6 +341,7 @@ class OptimizerTests(SimpleTestCase):
                     ('reviewer', models.ForeignKey('test_app.Reviewer', models.CASCADE)),
                 ]),
             ],
+            app_label='test_app',
         )
 
     def test_create_model_add_field(self):
@@ -360,7 +413,7 @@ class OptimizerTests(SimpleTestCase):
                     ('url', models.TextField()),
                     ('foo_fk', models.ForeignKey('migrations.Foo', models.CASCADE)),
                 ]),
-                migrations.AddField('Foo', 'bar_fk', models.ForeignKey('migrations.Foo', models.CASCADE)),
+                migrations.AddField('Foo', 'bar_fk', models.ForeignKey('migrations.Bar', models.CASCADE)),
             ],
         )
 
