@@ -59,14 +59,13 @@ class DateTimeFieldTests(TestCase):
         m1 = DateTimeModel.objects.create(d=d, dt=dt1, t=t)
         m2 = DateTimeModel.objects.create(d=d, dt=dt2, t=t)
         # In Vancouver, we expect both results.
-        self.assertQuerysetEqual(
+        self.assertCountEqual(
             DateTimeModel.objects.filter(dt__date=d),
-            [repr(m1), repr(m2)],
-            ordered=False
+            [m1, m2],
         )
         with self.settings(TIME_ZONE='UTC'):
             # But in UTC, the __date only matches one of them.
-            self.assertQuerysetEqual(DateTimeModel.objects.filter(dt__date=d), [repr(m1)])
+            self.assertCountEqual(DateTimeModel.objects.filter(dt__date=d), [m1])
 
 
 class ValidationTest(SimpleTestCase):
