@@ -1,8 +1,6 @@
-from unittest import skipIf
-
 from django import forms
-from django.db import connection, models
-from django.test import TestCase
+from django.db import models
+from django.test import SimpleTestCase, TestCase
 
 from .models import Post
 
@@ -32,8 +30,17 @@ class TextFieldTests(TestCase):
     def test_lookup_integer_in_textfield(self):
         self.assertEqual(Post.objects.filter(body=24).count(), 0)
 
-    @skipIf(connection.vendor == 'mysql', 'Running on MySQL requires utf8mb4 encoding (#18392)')
     def test_emoji(self):
         p = Post.objects.create(title='Whatever', body='Smile 😀.')
         p.refresh_from_db()
         self.assertEqual(p.body, 'Smile 😀.')
+
+
+class TestMethods(SimpleTestCase):
+    def test_deconstruct(self):
+        field = models.TextField()
+        *_, kwargs = field.deconstruct()
+        self.assertEqual(kwargs, {})
+        field = models.TextField(db_collation='utf8_esperanto_ci')
+        *_, kwargs = field.deconstruct()
+        self.assertEqual(kwargs, {'db_collation': 'utf8_esperanto_ci'})
