@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.test import TestCase
 
 from .models import Person
@@ -7,9 +5,9 @@ from .models import Person
 
 class PropertyTests(TestCase):
 
-    def setUp(self):
-        self.a = Person(first_name='John', last_name='Lennon')
-        self.a.save()
+    @classmethod
+    def setUpTestData(cls):
+        cls.a = Person.objects.create(first_name='John', last_name='Lennon')
 
     def test_getter(self):
         self.assertEqual(self.a.full_name, 'John Lennon')
@@ -20,7 +18,7 @@ class PropertyTests(TestCase):
             setattr(self.a, 'full_name', 'Paul McCartney')
 
         # And cannot be used to initialize the class.
-        with self.assertRaisesMessage(TypeError, "'full_name' is an invalid keyword argument"):
+        with self.assertRaisesMessage(TypeError, "Person() got an unexpected keyword argument 'full_name'"):
             Person(full_name='Paul McCartney')
 
         # But "full_name_2" has, and it can be used to initialize the class.
