@@ -12,8 +12,9 @@ from django.test import SimpleTestCase, override_settings, skipUnlessDBFeature
 from django.test.utils import Approximate
 
 from .models import (
-    Actor, Article, Author, AuthorProfile, BaseModel, Category, ComplexModel,
-    Movie, Player, ProxyBaseModel, ProxyProxyBaseModel, Score, Team,
+    Actor, Article, Author, AuthorProfile, BaseModel, Category, Child,
+    ComplexModel, Movie, Player, ProxyBaseModel, ProxyProxyBaseModel, Score,
+    Team,
 )
 
 
@@ -54,7 +55,7 @@ class SerializerRegistrationTests(SimpleTestCase):
             serializers.unregister_serializer("nonsense")
 
     def test_builtin_serializers(self):
-        "Requesting a list of serializer formats popuates the registry"
+        "Requesting a list of serializer formats populates the registry"
         all_formats = set(serializers.get_serializer_formats())
         public_formats = set(serializers.get_public_serializer_formats())
 
@@ -78,9 +79,8 @@ class SerializerRegistrationTests(SimpleTestCase):
             serializers.get_serializer("nonsense")
 
         # SerializerDoesNotExist is instantiated with the nonexistent format
-        with self.assertRaises(SerializerDoesNotExist) as cm:
+        with self.assertRaisesMessage(SerializerDoesNotExist, 'nonsense'):
             serializers.get_serializer("nonsense")
-        self.assertEqual(cm.exception.args, ("nonsense",))
 
     def test_get_unknown_deserializer(self):
         with self.assertRaises(SerializerDoesNotExist):
@@ -90,29 +90,30 @@ class SerializerRegistrationTests(SimpleTestCase):
 class SerializersTestBase:
     serializer_name = None  # Set by subclasses to the serialization format name
 
-    def setUp(self):
+    @classmethod
+    def setUpTestData(cls):
         sports = Category.objects.create(name="Sports")
         music = Category.objects.create(name="Music")
         op_ed = Category.objects.create(name="Op-Ed")
 
-        self.joe = Author.objects.create(name="Joe")
-        self.jane = Author.objects.create(name="Jane")
+        cls.joe = Author.objects.create(name='Joe')
+        cls.jane = Author.objects.create(name='Jane')
 
-        self.a1 = Article(
-            author=self.jane,
+        cls.a1 = Article(
+            author=cls.jane,
             headline="Poker has no place on ESPN",
             pub_date=datetime(2006, 6, 16, 11, 00)
         )
-        self.a1.save()
-        self.a1.categories.set([sports, op_ed])
+        cls.a1.save()
+        cls.a1.categories.set([sports, op_ed])
 
-        self.a2 = Article(
-            author=self.joe,
+        cls.a2 = Article(
+            author=cls.joe,
             headline="Time to reform copyright",
             pub_date=datetime(2006, 6, 16, 13, 00, 11, 345)
         )
-        self.a2.save()
-        self.a2.categories.set([music, op_ed])
+        cls.a2.save()
+        cls.a2.categories.set([music, op_ed])
 
     def test_serialize(self):
         """Basic serialization works."""
@@ -202,7 +203,7 @@ class SerializersTestBase:
         for field_name in valid_fields:
             self.assertTrue(self._get_field_values(serial_str, field_name))
 
-    def test_serialize_unicode(self):
+    def test_serialize_unicode_roundtrip(self):
         """Unicode makes the roundtrip intact"""
         actor_name = "Za\u017c\u00f3\u0142\u0107"
         movie_title = 'G\u0119\u015bl\u0105 ja\u017a\u0144'
@@ -219,6 +220,13 @@ class SerializersTestBase:
         mv_obj = obj_list[0].object
         self.assertEqual(mv_obj.title, movie_title)
 
+    def test_unicode_serialization(self):
+        unicode_name = 'יוניקוד'
+        data = serializers.serialize(self.serializer_name, [Author(name=unicode_name)])
+        self.assertIn(unicode_name, data)
+        objs = list(serializers.deserialize(self.serializer_name, data))
+        self.assertEqual(objs[0].object.name, unicode_name)
+
     def test_serialize_progressbar(self):
         fake_stdout = StringIO()
         serializers.serialize(
@@ -242,6 +250,19 @@ class SerializersTestBase:
         with self.assertNumQueries(0):
             serializers.serialize(self.serializer_name, [mv])
 
+    def test_serialize_prefetch_related_m2m(self):
+        # One query for the Article table and one for each prefetched m2m
+        # field.
+        with self.assertNumQueries(3):
+            serializers.serialize(
+                self.serializer_name,
+                Article.objects.all().prefetch_related('categories', 'meta_data'),
+            )
+        # One query for the Article table, and two m2m queries for each
+        # article.
+        with self.assertNumQueries(5):
+            serializers.serialize(self.serializer_name, Article.objects.all())
+
     def test_serialize_with_null_pk(self):
         """
         Serialized data with no primary key results
@@ -340,6 +361,14 @@ class SerializersTestBase:
         self.assertEqual(base_data, proxy_data.replace('proxy', ''))
         self.assertEqual(base_data, proxy_proxy_data.replace('proxy', ''))
 
+    def test_serialize_inherited_fields(self):
+        child_1 = Child.objects.create(parent_data='a', child_data='b')
+        child_2 = Child.objects.create(parent_data='c', child_data='d')
+        child_1.parent_m2m.add(child_2)
+        child_data = serializers.serialize(self.serializer_name, [child_1, child_2])
+        self.assertEqual(self._get_field_values(child_data, 'parent_m2m'), [])
+        self.assertEqual(self._get_field_values(child_data, 'parent_data'), [])
+
 
 class SerializerAPITests(SimpleTestCase):
 
