@@ -1,5 +1,4 @@
-from __future__ import unicode_literals
-
+from django.template import TemplateSyntaxError
 from django.test import SimpleTestCase
 
 from ...utils import setup
@@ -14,3 +13,9 @@ class GetAvailableLanguagesTagTests(SimpleTestCase):
     def test_i18n12(self):
         output = self.engine.render_to_string('i18n12')
         self.assertEqual(output, 'de')
+
+    @setup({'syntax_i18n': '{% load i18n %}{% get_available_languages a langs %}'})
+    def test_no_as_var(self):
+        msg = "'get_available_languages' requires 'as variable' (got ['get_available_languages', 'a', 'langs'])"
+        with self.assertRaisesMessage(TemplateSyntaxError, msg):
+            self.engine.render_to_string('syntax_i18n')
