@@ -93,6 +93,10 @@ class CachedLoaderTests(SimpleTestCase):
         """
         self.assertEqual(self.engine.template_loaders[0].cache_key(lazystr('template.html'), []), 'template.html')
 
+    def test_get_dirs(self):
+        inner_dirs = self.engine.template_loaders[0].loaders[0].get_dirs()
+        self.assertSequenceEqual(list(self.engine.template_loaders[0].get_dirs()), list(inner_dirs))
+
 
 class FileSystemLoaderTests(SimpleTestCase):
 
@@ -191,11 +195,12 @@ class FileSystemLoaderTests(SimpleTestCase):
             tmppath = os.path.join(tmpdir, tmpfile.name)
             os.chmod(tmppath, 0o0222)
             with self.set_dirs([tmpdir]):
-                with self.assertRaisesMessage(IOError, 'Permission denied'):
+                with self.assertRaisesMessage(PermissionError, 'Permission denied'):
                     self.engine.get_template(tmpfile.name)
 
     def test_notafile_error(self):
-        with self.assertRaises(IOError):
+        # Windows raises PermissionError when trying to open a directory.
+        with self.assertRaises(PermissionError if sys.platform == 'win32' else IsADirectoryError):
             self.engine.get_template('first')
 
 
