@@ -13,12 +13,14 @@ class OriginTestCase(TestCase):
         a = self.engine.get_template('index.html')
         b = self.engine.get_template('index.html')
         self.assertEqual(a.origin, b.origin)
-        self.assertTrue(a.origin == b.origin)
-        self.assertFalse(a.origin != b.origin)
+        # Use assertIs() to test __eq__/__ne__.
+        self.assertIs(a.origin == b.origin, True)
+        self.assertIs(a.origin != b.origin, False)
 
     def test_origin_compares_not_equal(self):
         a = self.engine.get_template('first/test.html')
         b = self.engine.get_template('second/test.html')
         self.assertNotEqual(a.origin, b.origin)
-        self.assertFalse(a.origin == b.origin)
-        self.assertTrue(a.origin != b.origin)
+        # Use assertIs() to test __eq__/__ne__.
+        self.assertIs(a.origin == b.origin, False)
+        self.assertIs(a.origin != b.origin, True)
