@@ -15,9 +15,6 @@ class A01(models.Model):
     class Meta:
         db_table = 'a01'
 
-    def __str__(self):
-        return self.f_a
-
 
 class B01(models.Model):
     fk_a = models.ForeignKey(A01, models.CASCADE)
@@ -29,9 +26,6 @@ class B01(models.Model):
         # 'managed' is True by default. This tests we can set it explicitly.
         managed = True
 
-    def __str__(self):
-        return self.f_a
-
 
 class C01(models.Model):
     mm_a = models.ManyToManyField(A01, db_table='d01')
@@ -41,8 +35,6 @@ class C01(models.Model):
     class Meta:
         db_table = 'c01'
 
-    def __str__(self):
-        return self.f_a
 
 # All of these models use the same tables as the previous set (they are shadows
 # of possibly a subset of the columns). There should be no creation errors,
@@ -56,9 +48,6 @@ class A02(models.Model):
         db_table = 'a01'
         managed = False
 
-    def __str__(self):
-        return self.f_a
-
 
 class B02(models.Model):
     class Meta:
@@ -69,9 +58,6 @@ class B02(models.Model):
     f_a = models.CharField(max_length=10, db_index=True)
     f_b = models.IntegerField()
 
-    def __str__(self):
-        return self.f_a
-
 
 # To re-use the many-to-many intermediate table, we need to manually set up
 # things up.
@@ -84,9 +70,6 @@ class C02(models.Model):
         db_table = 'c01'
         managed = False
 
-    def __str__(self):
-        return self.f_a
-
 
 class Intermediate(models.Model):
     a02 = models.ForeignKey(A02, models.CASCADE, db_column="a01_id")
