@@ -1,21 +1,22 @@
 import datetime
 import sys
 import unittest
+from pathlib import Path
 from unittest import mock
 from urllib.parse import quote_plus
 
 from django.test import SimpleTestCase
 from django.utils.encoding import (
     DjangoUnicodeDecodeError, escape_uri_path, filepath_to_uri, force_bytes,
-    force_text, get_system_encoding, iri_to_uri, repercent_broken_unicode,
-    smart_bytes, smart_text, uri_to_iri,
+    force_str, get_system_encoding, iri_to_uri, repercent_broken_unicode,
+    smart_bytes, smart_str, uri_to_iri,
 )
 from django.utils.functional import SimpleLazyObject
 from django.utils.translation import gettext_lazy
 
 
 class TestEncodingUtils(SimpleTestCase):
-    def test_force_text_exception(self):
+    def test_force_str_exception(self):
         """
         Broken __str__ actually raises an error.
         """
@@ -25,19 +26,19 @@ class TestEncodingUtils(SimpleTestCase):
 
         # str(s) raises a TypeError if the result is not a text type.
         with self.assertRaises(TypeError):
-            force_text(MyString())
+            force_str(MyString())
 
-    def test_force_text_lazy(self):
+    def test_force_str_lazy(self):
         s = SimpleLazyObject(lambda: 'x')
-        self.assertIs(type(force_text(s)), str)
+        self.assertIs(type(force_str(s)), str)
 
-    def test_force_text_DjangoUnicodeDecodeError(self):
+    def test_force_str_DjangoUnicodeDecodeError(self):
         msg = (
             "'utf-8' codec can't decode byte 0xff in position 0: invalid "
             "start byte. You passed in b'\\xff' (<class 'bytes'>)"
         )
         with self.assertRaisesMessage(DjangoUnicodeDecodeError, msg):
-            force_text(b'\xff')
+            force_str(b'\xff')
 
     def test_force_bytes_exception(self):
         """
@@ -76,16 +77,16 @@ class TestEncodingUtils(SimpleTestCase):
         self.assertEqual(smart_bytes(1), b'1')
         self.assertEqual(smart_bytes('foo'), b'foo')
 
-    def test_smart_text(self):
+    def test_smart_str(self):
         class Test:
             def __str__(self):
                 return 'ŠĐĆŽćžšđ'
 
         lazy_func = gettext_lazy('x')
-        self.assertIs(smart_text(lazy_func), lazy_func)
-        self.assertEqual(smart_text(Test()), '\u0160\u0110\u0106\u017d\u0107\u017e\u0161\u0111')
-        self.assertEqual(smart_text(1), '1')
-        self.assertEqual(smart_text('foo'), 'foo')
+        self.assertIs(smart_str(lazy_func), lazy_func)
+        self.assertEqual(smart_str(Test()), '\u0160\u0110\u0106\u017d\u0107\u017e\u0161\u0111')
+        self.assertEqual(smart_str(1), '1')
+        self.assertEqual(smart_str('foo'), 'foo')
 
     def test_get_default_encoding(self):
         with mock.patch('locale.getdefaultlocale', side_effect=Exception):
@@ -104,8 +105,10 @@ class TestEncodingUtils(SimpleTestCase):
 class TestRFC3987IEncodingUtils(unittest.TestCase):
 
     def test_filepath_to_uri(self):
-        self.assertEqual(filepath_to_uri(None), None)
+        self.assertIsNone(filepath_to_uri(None))
         self.assertEqual(filepath_to_uri('upload\\чубака.mp4'), 'upload/%D1%87%D1%83%D0%B1%D0%B0%D0%BA%D0%B0.mp4')
+        self.assertEqual(filepath_to_uri(Path('upload/test.png')), 'upload/test.png')
+        self.assertEqual(filepath_to_uri(Path('upload\\test.png')), 'upload/test.png')
 
     def test_iri_to_uri(self):
         cases = [
@@ -121,10 +124,11 @@ class TestRFC3987IEncodingUtils(unittest.TestCase):
         ]
 
         for iri, uri in cases:
-            self.assertEqual(iri_to_uri(iri), uri)
+            with self.subTest(iri):
+                self.assertEqual(iri_to_uri(iri), uri)
 
-            # Test idempotency.
-            self.assertEqual(iri_to_uri(iri_to_uri(iri)), uri)
+                # Test idempotency.
+                self.assertEqual(iri_to_uri(iri_to_uri(iri)), uri)
 
     def test_uri_to_iri(self):
         cases = [
@@ -144,10 +148,11 @@ class TestRFC3987IEncodingUtils(unittest.TestCase):
         ]
 
         for uri, iri in cases:
-            self.assertEqual(uri_to_iri(uri), iri)
+            with self.subTest(uri):
+                self.assertEqual(uri_to_iri(uri), iri)
 
-            # Test idempotency.
-            self.assertEqual(uri_to_iri(uri_to_iri(uri)), iri)
+                # Test idempotency.
+                self.assertEqual(uri_to_iri(uri_to_iri(uri)), iri)
 
     def test_complementarity(self):
         cases = [
@@ -165,13 +170,19 @@ class TestRFC3987IEncodingUtils(unittest.TestCase):
         ]
 
         for uri, iri in cases:
-            self.assertEqual(iri_to_uri(uri_to_iri(uri)), uri)
-            self.assertEqual(uri_to_iri(iri_to_uri(iri)), iri)
+            with self.subTest(uri):
+                self.assertEqual(iri_to_uri(uri_to_iri(uri)), uri)
+                self.assertEqual(uri_to_iri(iri_to_uri(iri)), iri)
 
     def test_escape_uri_path(self):
-        self.assertEqual(
-            escape_uri_path('/;some/=awful/?path/:with/@lots/&of/+awful/chars'),
-            '/%3Bsome/%3Dawful/%3Fpath/:with/@lots/&of/+awful/chars'
-        )
-        self.assertEqual(escape_uri_path('/foo#bar'), '/foo%23bar')
-        self.assertEqual(escape_uri_path('/foo?bar'), '/foo%3Fbar')
+        cases = [
+            (
+                '/;some/=awful/?path/:with/@lots/&of/+awful/chars',
+                '/%3Bsome/%3Dawful/%3Fpath/:with/@lots/&of/+awful/chars',
+            ),
+            ('/foo#bar', '/foo%23bar'),
+            ('/foo?bar', '/foo%3Fbar'),
+        ]
+        for uri, expected in cases:
+            with self.subTest(uri):
+                self.assertEqual(escape_uri_path(uri), expected)
